/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  cfg.cpp
 */
#include "cfg.hpp"
#include <iostream>
#include <fstream>
#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <algorithm>
#include <iterator>

namespace
{

  /*!
   *  \brief  gp@̕\
   */
  void help_message()
  {
    static const char message[] =
      "Usage: cfg [options] file\n"
      "Options:\n"
      "  -h   --help              Display this information\n"
      "  -lj  --japanese          Use Japanese\n"
      "  -le  --english           Use English\n"
      "  -cfg --kernel_cfg=<file> Specify the <file> instead of `kernel_cfg.c\'\n"
      "  -id  --kernel_id=<file>  Specify the <file> instead of `kernel_id.h\'\n"
      "  -ucn                     Enable universal character name in CFG file\n"
      "  -sjis                    Enable Shift-JIS in CFG file\n"
      "  -n <directory>           Specify the <directory> where output files\n"
      "  -kernel <type>           Specify the kernel type, <type> which is JSP or FDMP\n";
    
    std::cerr << _( message ) << std::endl;
  }

  /*!
   *  \brief  bZ[W̑I
   *  \param  options R}hCIvV
   *
   *  o[Wł͉pƓ{̂ݑΉĂ܂B
   *  ܂AIɉpw肷IvVł-le--english͖A
   *  -lj܂--japanesew肳ȂΉpɂȂ܂B
   */
  void select_language( const toppers::cmdline::option_array& options )
  {
    if ( options.find( "-lj" ) )
    {
      set_language( toppers::reporter::Japanese );
    }
  }

  /*!
   *  \brief  R}hC̉
   *  \param  argc  main֐̑
   *  \param  argv  main֐̑
   *  \param  opts  IvV̊i[
   *  \return IvVȊÖijԂ
   *  \attention  ̊֐͓̓t@Cꍇ̓G[ɂ܂B
   */
  std::auto_ptr<std::string> parse_cmdline( int argc, char* argv[], toppers::cmdline::option_array& opts )
  {
    using namespace toppers;
    using namespace toppers::itron;

    cmdline cmdl( argc, argv );
    cmdline::option_array options;
    const char pattern[] = "-h --help "
                           "-lj --japanese "
                           "-le --english "
                           "-cfg: --kernel_cfg= "
                           "-id: --kernel_id= "
                           "-cpu: "
                           "-system: "
                           "-tool: "
                           "-obj "
                           "-ucn "
                           "-sjis "
                           "-n: "
                           "-kernel: "
                           "-s: --source= "
                           "-iapi --ignore-api "
                           "-t --through "
                           "-obj: --dump-object= "
                           "-ao: --assign-order= "
                           "-var --variable-id "
                           "-il --include-libstyle "
                           "-oproto --output-prototype "
                           "-z -nonzero "
                           "-c --check "
                           "-x: "
                           "-debug";
    const std::auto_ptr<std::vector<std::string> > sources( cmdl.parse( pattern, options ) );

    options.remap_option( "--help", "-h" );
    options.remap_option( "--japanese", "-lj" );
    options.remap_option( "--english", "-le" );
    options.remap_option( "--kernel_cfg=", "-cfg:" );
    options.remap_option( "--kernel_id=", "-id:" );
    options.remap_option( "--source=", "-s:" );
    options.remap_option( "--ignore-api", "-iapi" );
    options.remap_option( "--through", "-t" );
    options.remap_option( "--dump-object=", "-obj:" );
    options.remap_option( "--assign-order=", "-ao:" );
    options.remap_option( "--variable-id", "-var" );
    options.remap_option( "--include-libstyle", "-il" );
    options.remap_option( "--output-prototype", "-oproto" );
    options.remap_option( "--nonzero", "-z" );
    options.remap_option( "--check", "-c" );

    select_language( options );

    if ( options.find( "-h" ) )
    {
      help_message();
      toppers::reporter::terminator x;
      throw x;
    }

    std::vector<std::string> t;
    if ( options.find( "-s", t ) )
    {
      std::copy( t.begin(), t.end(), std::back_inserter( *sources ) );
    }

    if ( !sources->empty() )
    {
      std::vector<std::string>::const_iterator iter( cmdline::find_illegal_options( sources->begin(), sources->end() ) );
      if ( iter == sources->end() )
      {
        if ( sources->size() > 1 )
        {
          stop( _( "too many input files" ) );  // ̓t@C
        }
      }
      else
      {
        toppers::error( _( "illegal option `%1%\'" ) % *iter ); // sȃIvV
      }
    }

    std::auto_ptr<std::string> source;
    if ( !sources->empty() )
    {
      source.reset( new std::string( sources->front() ) );
    }
    opts.swap( options );
    return source;
  }

  /*!
   *  \brief  #pragmaw߂̉
   *  \param  pragmas #pragmaw߂̃p[^
   *  \prama  options R}hCIvV
   */
  void analyze_pragmas( const std::vector<toppers::line_buf>& pragmas, toppers::cmdline::option_array& options )
  {
    typedef std::vector<toppers::line_buf>::const_iterator const_iterator;
    for ( const_iterator iter( pragmas.begin() ), last( pragmas.end() ); iter != last; ++iter )
    {
      using namespace boost::spirit;
      std::string kernel_type;
      if ( parse( iter->buf_.begin(), iter->buf_.end(),
                  str_p( "kernel" ) >> '(' >> '\"' >> ( *( c_escape_ch_p - '\"' ) )[ assign_a( kernel_type ) ] >> '\"' >> ')',
                  space_p ).full )
      {
        if ( !options.replace( "-kernel", kernel_type ) )
        {
          options.insert( "-kernel", kernel_type );
        }
        break;
      }
      else
      {
        toppers::warning( _( "unknown #pragma directive `%1%\'" ) % iter->buf_ );
      }
    }
  }

  /*!
   *  \brief  \[XR[h̐
   *  \param  factory IuWFNgt@Ng
   *  \param  data    OILt@C͍̉ς݃f[^
   */
  void generate_files( const boost::shared_ptr<toppers::itron::cfg_factory>& factory, const boost::shared_ptr<toppers::itron::cfg_contents>& contents )
  {
    typedef std::vector<boost::shared_ptr<toppers::itron::cfg_contents> > vector_t;
    boost::shared_ptr<vector_t> sub_contents_list( contents->classify() );

    if ( sub_contents_list->empty() )
    {
      toppers::error( _( "no contents in cfg file" ) );
    }

    for ( vector_t::const_iterator iter( sub_contents_list->begin() ), last( sub_contents_list->end() ); iter != last; ++iter )
    {
      factory->set_contents( *iter );

      boost::shared_ptr<toppers::kernel_cfg> k_cfg( factory->create_kernel_cfg() );
      static_cast<toppers::itron::cfg_kernel_cfg*>( k_cfg.get() )->load( *iter );
      k_cfg->save();

      try
      {
        boost::shared_ptr<toppers::kernel_id> k_id( factory->create_kernel_id() );
        static_cast<toppers::itron::cfg_kernel_id*>( k_id.get() )->load( *iter );
        k_id->save();

        if ( factory->options()->find( "-c" ) )
        {
          try
          {
            boost::shared_ptr<toppers::itron::kernel_chk> k_chk( factory->create_kernel_chk() );
            k_chk->load( *iter );
            k_chk->save();
          }
          catch ( ... )
          {
            std::remove( k_id->file().c_str() );
            throw;
          }
        }
      }
      catch ( ... )
      {
        std::remove( k_cfg->file().c_str() );
        throw;
      }
    }
  }

  /*!
   *  \brief  RtBM[^̃C
   *  \param  argc  main֐̑
   *  \param  argv  main֐̑
   */
  int cfg_main( int argc, char** argv )
  {
    using namespace toppers;
    using namespace toppers::itron;
    using namespace boost::spirit;

    // R}hC̉
    cmdline::option_array options;
    std::auto_ptr<std::string> source( parse_cmdline( argc, argv, options ) );

    // O
    std::istream* istr = &std::cin;
    std::ifstream ifstr;
    text txt;
    c_pp_line<text::container> pp;

    if ( source.get() != 0 )
    {
      ifstr.open( source->c_str() );
      if ( !ifstr.is_open() )
      {
        error( _( "cannot open file `%1%\'" ) % *source );
      }
      istr = &ifstr;
      std::string t( "#line 1 \"" + *source + "\"\n" );
      txt.assign( t.begin(), t.end(), pp );
    }
    txt.append( *istr, pp );
    analyze_pragmas( pp.pragmas(), options );

    std::string lang;
    if ( options.find( "-x", lang ) && lang == "c++" )
    {
      if ( !options.find( "-cfg" ) )
      {
        options.insert( "-cfg", "kernel_cfg.cpp" );
      }
    }

    // t@Ng𐶐
    std::string kernel_type( "JSP" );
    options.find( "-kernel", kernel_type );
    boost::shared_ptr<cfg_factory> factory( cfg_factory::build( options, kernel_type ) );
    if ( !factory )
    {
      toppers::error( _( "kerne type `%1%\' is not supported" ) % kernel_type );
    }
    cfg_factory::global( factory );

    // \
    codeset_t codeset = euc_jp;
    if ( options.find( "-sjis" ) )
    {
      codeset = shift_jis;
    }
    bool ucn = false;
    if ( options.find( "-ucn" ) )
    {
      ucn = true;
    }
    text::const_iterator next( txt.begin() );
    boost::shared_ptr<cfg_contents> contents( factory->create_cfg_contents( next, txt.end(), ucn, codeset ) );
    if ( !contents )
    {
      return EXIT_FAILURE;
    }

    // t@C̐
    generate_files( factory, contents );

    return EXIT_SUCCESS;
  }

}

int main( int argc, char** argv )
{
  int status;
  try
  {
    cfg_init();
    status = cfg_main( argc, argv );
    if ( status != EXIT_SUCCESS )
    {
      toppers::error( _( "unknown error" ) );
    }
  }
  catch ( toppers::reporter::terminator& )
  {
    status = EXIT_FAILURE;
  }
  catch ( std::exception& e )
  {
    std::fputs( e.what(), stderr );
    std::fputc( '\n', stderr );
    status = EXIT_FAILURE;
  }
  catch ( ... )
  {
    toppers::stop( _( "internal error" ) );
    status = EXIT_FAILURE;
  }
  return status;
}

