/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2004 by Witz Corporation, JAPAN    
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  translate.cpp
 */
#include "toppers/reporter.hpp"
#include <cstring>
#include <string>
#include <map>
#include <utility>
#include <boost/scoped_array.hpp>

#ifdef  TOPPERS_HAS_ICONV
#include <iconv.h>
#endif

namespace
{

  const int number_of_languages = toppers::reporter::number_of_languages;
  toppers::reporter::language_type language = toppers::reporter::English;

  const char* message_table[][number_of_languages] =
  {
    {
      "Usage: cfg [options] file\n"
      "Options:\n"
      "  -h   --help              Display this information\n"
      "  -lj  --japanese          Use Japanese\n"
      "  -le  --english           Use English\n"
      "  -cfg --kernel_cfg=<file> Specify the <file> instead of `kernel_cfg.c\'\n"
      "  -id  --kernel_id=<file>  Specify the <file> instead of `kernel_id.h\'\n"
      "  -ucn                     Enable universal character name in CFG file\n"
      "  -sjis                    Enable Shift-JIS in CFG file\n"
      "  -n <directory>           Specify the <directory> where output files\n"
      "  -kernel <type>           Specify the kernel type, <type> which is JSP or FDMP\n",

      "gp@: cfg [options] file\n"
      "IvV:\n"
      "  -h  --help               ̏\܂\n"
      "  -lj  --japanese          {gp܂\n"
      "  -le  --english           pgp܂\n"
      "  -cfg --kernel_cfg=<file> `kernel_cfg.c\'̑<file>w肵܂\n"
      "  -id  --kernel_id=<file>  `kernel_id.h\'̑<file>w肵܂\n"
      "  -ucn                     CFGt@C̍ە܂\n"
      "  -sjis                    CFGt@C̃VtgJIS܂\n"
      "  -n <directory>           t@C̏o͐Ƃ<directory>w肵܂\n"
      "  -kernel <type>           J[lʁiJSP܂FDMPjw肵܂\n",
    },
    {
      "no input file",
      "̓t@Cw肳Ă܂",
    },
    {
      "too many input files",
      "̓t@C܂",
    },
    {
      "illegal option `%1%\'",
      "`%1%\'͕sȃIvVł",
    },
    {
      "unknown error",
      "m̃G[܂",
    },
    {
      "internal error",
      "G[܂",
    },
    {
      "argument to `%1%\' is missing",
      "`%1%\'̈܂",
    },
    {
      "%1% is not directory",
      "%1%̓fBNgł͂܂",
    },
    {
      "cannot make directory `%1%\'",
      "fBNg`%1%\'쐬ł܂",
    },
    {
      "cannot create file `%1%\'",
      "t@C`%1%\'쐬ł܂",
    },
    {
      "cannot open file `%1%\'",
      "t@C`%1%\'J܂",
    },
    {
      "warning: ",
      "x: ",
    },
    {
      "error: ",
      "G[: ",
    },
    {
      "fatal error: ",
      "vIG[: ",
    },
    {
      "syntax error",
      "\G[",
    },
    {
      "static API `%1%\' cannot be unresolved",
      "ÓIAPI `%1%\'ł܂",
    },
    {
      "parameter `%1%\' must be `NULL\'",
      "p[^`%1%\'NULLłȂ΂Ȃ܂",
    },
    {
      "parameter `%1%\' is negative",
      "p[^`%1%\'̒lł",
    },
    {
      "`INCLUDE\' expects string literal",
      "`INCLUDE\'ɂ͕񃊃ew肵Ȃ΂Ȃ܂",
    },
    {
      "missing `%1%\' before `%2%\'",
      "`%2\'̑O`%1%\'܂",
    },
    {
      "illegal argument `%1%'",
      "`%1\'sł",
    },
    {
      "id `%1%\' of %2% is duplicated",
      "%2%IDԍ`%1\'dĂ܂",
    },
    {
      "missing `}\'",
        "`}\'܂",
    },
    {
      "kerne type `%1%\' is not supported",
      "J[l`%1%\'̓T|[gĂ܂"
    },
    {
      "unknown #pragma directive `%1%\'",
        "s#pragmaw`%1%\'",
    },
    {
      "no contents in cfg file",
      "cfgt@CɃRec܂܂Ă܂",
    },
  };

  const char* encode( const char* s )
  {
#if defined( __CYGWIN__ ) || defined( __MINGW32__ )
# if '@' == 0xa1a1   // CFG̃\[XR[hEUC
    static boost::scoped_array<char> t;
    std::size_t ss = std::strlen( s );
    std::size_t st = ss;

    t.reset( new char[ss+1] );
    const char *ps = s;
    char *pt = t.get();
    iconv_t cd = iconv_open( "SHIFT_JIS", "EUCJP" );
    iconv( cd, &ps, &ss, &pt, &st );
    iconv_close( cd );
    *pt = '\0';
    s = t.get();
# endif
#endif
    return s;
  }

  const char* translate( const char* s )
  {
    struct translation_map
    {
      std::map<std::string, const char*const*> map_;

      translation_map( const char* message[][number_of_languages], std::size_t size )
      {
        for ( std::size_t i = 0; i < size; i++ )
        {
          map_.insert( std::make_pair( std::string( message[i][0] ), message[i] ) );
        }
      }
    };
    static translation_map tr( message_table, sizeof( message_table )/sizeof( message_table[0] ) );
    const std::map<std::string, const char*const*>::const_iterator iter( tr.map_.find( s ) );
    if ( iter != tr.map_.end() )
    {
      return encode( iter->second[language] );
    }
    return s;
  }

}

/*!
 *  \brief  bZ[W̐ݒ
 *  \param  lang  ݒ肷錾itoppers::reporter::Japanse or toppers::reporter::Englishj
 */
void set_language( toppers::reporter::language_type lang )
{
  language = lang;
  toppers::reporter::set_translator( &translate );
}

