/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  chk.cpp
 */
#include "chk.hpp"
#include <iostream>
#include <fstream>
#include <cstdlib>

namespace
{

  /*!
   *  \brief  gp@̕\
   */
  void help_message()
  {
    static const char message[] =
      "Usage: cfg [options] file\n"
      "Options:\n"
      "  -h   --help              Display this information\n"
      "  -lj  --japanese          Use Japanese\n"
      "  -le  --english           Use English\n"
      "  -m   --module=<file>     Specify the <file> for a load module\n"
      "  -cs  --script=<file>     Specify the <file> instead of `kernel_chk.srec\'\n"
      "\n";
    
    std::cerr << _( message ) << std::endl;
  }

  /*!
   *  \brief  bZ[W̑I
   *  \param  options R}hCIvV
   *
   *  o[Wł͉pƓ{̂ݑΉĂ܂B
   *  ܂AIɉpw肷IvVł-le--english͖A
   *  -lj܂--japanesew肳ȂΉpɂȂ܂B
   */
  void select_language( const toppers::cmdline::option_array& options )
  {
    if ( options.find( "-lj" ) )
    {
      set_language( toppers::reporter::Japanese );
    }
  }

  /*!
   *  \brief  R}hC̉
   *  \param  argc  main֐̑
   *  \param  argv  main֐̑
   *  \param  opts  IvV̊i[
   *  \return IvVȊÖijԂ
   *  \attention  ̊֐͓̓t@Cꍇ̓G[ɂ܂B
   */
  std::auto_ptr<std::string> parse_cmdline( int argc, char* argv[], toppers::cmdline::option_array& opts )
  {
    using namespace toppers;
    using namespace toppers::itron;

    cmdline cmdl( argc, argv );
    cmdline::option_array options;
    const char pattern[] = "-h --help "
                           "-lj --japanese "
                           "-le --english "
                           "-m: --module= "
                           "-cs: --script= "
                           "-cpu: "
                           "-system: "
                           "-tool: "
                           "-obj "
                           "-debug";
    std::auto_ptr<std::vector<std::string> > sources( cmdl.parse( pattern, options ) );

    options.remap_option( "--help", "-h" );
    options.remap_option( "--japanese", "-lj" );
    options.remap_option( "--english", "-le" );

    select_language( options );

    if ( options.find( "-h" ) )
    {
      help_message();
      toppers::reporter::terminator x;
      throw x;
    }

    std::vector<std::string> t;
    if ( options.find( "-s", t ) )
    {
      std::copy( t.begin(), t.end(), std::back_inserter( *sources ) );
    }

    if ( !sources->empty() )
    {
      std::vector<std::string>::const_iterator iter( cmdline::find_illegal_options( sources->begin(), sources->end() ) );
      if ( iter == sources->end() )
      {
        if ( sources->size() > 1 )
        {
          stop( _( "too many input files" ) );  // ̓t@C
        }
      }
      else
      {
        error( _( "illegal option `%1%\'" ) % *iter ); // sȃIvV
      }
    }

    std::auto_ptr<std::string> source;
    if ( !sources->empty() )
    {
      source.reset( new std::string( sources->front() ) );
    }
    opts.swap( options );
    return source;
  }

  /*!
   *  \brief  `FbJ[̃C
   *  \param  argc  main֐̑
   *  \param  argv  main֐̑
   */
  int chk_main( int argc, char** argv )
  {
    using namespace toppers;
    using namespace toppers::itron;

    // R}hC̉
    cmdline::option_array options;
    std::auto_ptr<std::string> source( parse_cmdline( argc, argv, options ) );

    // "kernel_chk.srec"̓ǂݍ
    std::string script( "kernel_chk.srec" );
    options.find( "-cs", script );
    std::ifstream ifs( script.c_str() );
    if ( !ifs.is_open() )
    {
      error( _( "cannot open file `%1%\'" ) % script );
    }
    s_record chk_srec( ifs );

    // t@Ng̐
    long prid = kernel_chk::read_prid( chk_srec );
    boost::shared_ptr<chk_factory> factory( chk_factory::build( options, prid ) );
    chk_factory::global( factory );

    boost::shared_ptr<kernel_chk> k_chk( factory->create_kernel_chk() );
    k_chk->load( chk_srec );

    // "jsp.syms""jsp.srec"̃t@C擾
    std::string symbols( "jsp.syms" );
    std::string load_module( "jsp.srec" );
    std::string opt_m;
    if ( options.find( "-m", opt_m ) )
    {
      std::string::size_type pos = opt_m.find( ',' );
      symbols = opt_m.substr( 0, pos );
      if ( pos != std::string::npos )
      {
        load_module = opt_m.substr( pos+1 );
      }
    }

    // "jsp.syms""jsp.srec"̓ǂݍ
    std::ifstream ifs_srec( load_module.c_str() );
    s_record lm_srec( ifs_srec );
    std::ifstream ifs_syms( symbols.c_str() );
    nm_symbol syms( ifs_syms );

    // eJ[lIuWFNg̃p[^`FbN
    boost::shared_ptr<std::vector<chk_factory::checker_type> > checkers( factory->get_checkers() );
    typedef std::vector<chk_factory::checker_type>::const_iterator checker_iterator;
    for ( checker_iterator iter( checkers->begin() ), last( checkers->end() ); iter != last; ++iter )
    {
      if ( !( *iter )( *k_chk, lm_srec, syms ) )
      {
        return EXIT_FAILURE;
      }
    }

    std::cout << "check complete" << std::endl;

    return EXIT_SUCCESS;
  }

}

int main( int argc, char** argv )
{
  int status;
  try
  {
    chk_init();
    status = chk_main( argc, argv );
    if ( status != EXIT_SUCCESS )
    {
      toppers::error( _( "unknown error" ) );
    }
  }
  catch ( toppers::reporter::terminator& )
  {
    status = EXIT_FAILURE;
  }
  catch ( std::exception& e )
  {
    std::fputs( e.what(), stderr );
    std::fputc( '\n', stderr );
    status = EXIT_FAILURE;
  }
  catch ( ... )
  {
    toppers::stop( _( "internal error" ) );
    status = EXIT_FAILURE;
  }
  return status;
}
