/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa  
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*!
 *  \file   toppers/c_pp_line.hpp
 *  \brief  #linew߂Ɋւ錾`
 */
#ifndef TOPPERS_C_PP_LINE_HPP_
#define TOPPERS_C_PP_LINE_HPP_

#include "toppers/text_line.hpp"
#include "toppers/parser.hpp"
#include <vector>
#include <functional>
#include <boost/shared_ptr.hpp>

namespace toppers
{

  namespace detail
  {

    struct c_pp_line_parser : boost::spirit::grammar<c_pp_line_parser>
    {
      template <class Scanner>
      struct definition
      {
        typedef boost::spirit::rule<Scanner> rule_t;
        rule_t r;
        definition( const c_pp_line_parser& self )
        {
          using namespace boost::spirit;
          r = (
                '#' >> lexeme_d[ str_p( "line" ) >> space_p >> uint_p[ assign_a( self.line_ ) ] ] >>
                c_strlit_parser( self.codeset_ )[ assign_a( self.file_ ) ]
              )
            | (
                '#' >>
                uint_p[ assign_a( self.line_ ) ] >>
                c_strlit_parser( self.codeset_ )[ assign_a( self.file_ ) ] >>
                *anychar_p
              );
        }
        const rule_t& start() const { return r; }
      };

      c_pp_line_parser( long& line, std::string& file, codeset_t codeset = ascii )
        : line_( line ), file_( file ), codeset_( codeset )
      {
      }

      long& line_;
      std::string& file_;
      codeset_t codeset_;
    };

    struct c_pp_pragma_parser : boost::spirit::grammar<c_pp_pragma_parser>
    {
      template <class Scanner>
      struct definition
      {
        typedef boost::spirit::rule<Scanner> rule_t;
        rule_t r;
        definition( const c_pp_pragma_parser& self )
        {
          using namespace boost::spirit;
          r = '#' >> lexeme_d[ str_p( "pragma" ) >> space_p >> ( +anychar_p )[ assign_a( self.parameter_ ) ] ];
        }
        const rule_t& start() const { return r; }
      };

      c_pp_pragma_parser( std::string& parameter ) : parameter_( parameter )
      {
      }
      std::string& parameter_;
    };

  }

  /*!
   *  \class  c_pp_line c_pp_line.hpp "toppers/c_pp_line.hpp"
   *  \brief  #linew߂邽߂̃t@N^NX
   *
   *  ̃NX basic_text NXƑgݍ킹Ďgp܂B
   */
  template <class Container>
  class c_pp_line : public std::binary_function<Container, line_buf, void>
  {
  public:
    typedef Container conatiner;

    /*!
     *  \brief  RXgN^
     *  \param  codeset R[hw
     */
    explicit c_pp_line( codeset_t codeset = ascii )
      : codeset_( codeset ), pragmas_( new std::vector<line_buf> )
    {
    }
    /*!
     *  \brief  ʉZq
     *  \param  cont  line_buf vfƂRei
     *  \param  buf   1sobt@
     */
    void operator()( conatiner& cont, line_buf& buf )
    {
      using namespace boost::spirit;
      long line;
      std::string file;
      detail::c_pp_line_parser c_pp_line_p( line, file, codeset_ );

      if ( parse( buf.buf_.begin(), buf.buf_.end(), c_pp_line_p, space_p ).full ) // #linew߂̏
      {
        using namespace boost::filesystem;
        buf.line_.line_ = line;
        assert( file.size() >= 2 );
        try
        {
          std::string file_name( file.substr( 1, file.size()-2 ) );
          buf.line_.file_ = path( file_name, native );
        }
        catch ( ... )
        {
          // GNŨvvZbT̓t@Cɑ镔<built-n><command line>
          // ƂȂꍇ邽߁AO\
          buf.line_.file_ = path( "unknown", native );
        }
      }
      else
      {
        std::string param;
        detail::c_pp_pragma_parser c_pp_pragma_p( param );

        if ( parse( buf.buf_.begin(), buf.buf_.end(), c_pp_pragma_p, space_p ).full ) // #pragmaw߂̏
        {
          line_buf t( buf );
          t.buf_ = param;
          pragmas_->push_back( t );
        }
        else
        {
          std::string::size_type pos = buf.buf_.find_first_not_of( " \t" );
          if ( pos == std::string::npos || buf.buf_[pos] != '#' )
          {
            cont.push_back( buf );
          }
          ++buf.line_.line_;
        }
      }
      buf.buf_.clear();
    }
    /*!
     *  \brief  #pragmaw߃Xg̎擾
     *  \return #pragmaw߃XgԂ
     */
    const std::vector<line_buf>& pragmas() const { return *pragmas_; }
  private:
    codeset_t codeset_;
    boost::shared_ptr<std::vector<line_buf> > pragmas_;
  };

}

#endif  // ! TOPPERS_C_PP_LINE_HPP_
