/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2004 by Witz Corporation, JAPAN  
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  toppers/factory.cpp
 */
#include "toppers/factory.hpp"
#include "toppers/reporter.hpp"
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/exception.hpp>

namespace toppers
{

  /*!
    *  \brief  RXgN^
    *  \param  options R}hCIvV
    *  \param  version o[W
    */
  factory::factory( const cmdline::option_array& options, const std::string& version )
    : options_( new cmdline::option_array( options ) ), version_( version )
  {
  }

  /*!
   *  \brief  kernel_idIuWFNg̐
   *  \return kernel_id̃CX^XwX}[g|C^Ԃ
   */
  const boost::shared_ptr<kernel_id> factory::create_kernel_id() const
  {
    return do_create_kernel_id( get_cfg_file_name( "-id", "kernel_id.h" ) );
  }

  /*!
   *  \brief  kernel_cfgIuWFNg̐
   *  \return kernel_cfg̃CX^XwX}[g|C^Ԃ
   */
  const boost::shared_ptr<kernel_cfg> factory::create_kernel_cfg() const
  {
    std::string file( get_cfg_file_name( "-cfg", "kernel_cfg.c" ) );

    // get_cfg_file_name gƁA-nIvVɂfBNgtĂ̂ŁA
    // ł͎gpłȂ
    std::string id_file( "kernel_id.h" );
    options_->find( "-id", id_file );

    return do_create_kernel_cfg( file, id_file );
  }

  /*!
    *  \brief  kernel_cfg.cp̐֐Q̐
    *  \return kernel_cfg.cp̐֐QwX}[g|C^Ԃ
    */
  const boost::shared_ptr<std::vector<factory::kernel_cfg_generator_type> > factory::create_kernel_cfg_generators() const
  {
    return do_create_kernel_cfg_generators();
  }

  /*!
   *  \brief  kernel_id.hp̐֐Q̐
   *  \return kernel_id.hp̐֐QwX}[g|C^Ԃ
   */
  const boost::shared_ptr<std::vector<factory::kernel_id_generator_type> > factory::create_kernel_id_generators() const
  {
    return do_create_kernel_id_generators();
  }

  /*!
   *  \brief  R}hCIvV
   *  \return R}hCIvVԂ
   */
  const boost::shared_ptr<cmdline::option_array>& factory::options() const
  {
    return options_;
  }

  /*!
   *  \brief  o[W
   *  \return o[WԂ
   */
  const std::string& factory::version() const
  {
    return version_;
  }

  /*!
   *  \brief  ݒt@C̎擾
   *  \param  option  t@Cw肷IvViex. "-cfg="j
   *  \param  default_name  IvVɂw肪Ȃꍇ̊薼
   *  \return ݒt@CԂ
   *
   *  `kernel_id.h'`kernel_cfg.c'Ȃǂ̐ݒt@Cio̓t@Cĵ̖擾܂B
   *  ݒt@C̓R}hCIvVɂĒڎwłقAo̓fBNg
   *  w肳邱ƂŃt@CipXjς܂B\n
   *  ̊֐ł́A𔽉fݒt@CԂ܂B
   */
  const std::string factory::get_cfg_file_name( const std::string& option, const std::string& default_name ) const
  {
    std::string file;
    options_->find( option, file );
    if ( file.empty() )
    {
      file = default_name;
    }
    return get_cfg_file_name( file );
  }

  /*!
   *  \brief  ݒt@C̎擾
   *  \param  file  t@C
   *  \return ݒt@CԂ
   */
  const std::string factory::do_get_cfg_file_name( const std::string& file ) const
  {
    boost::filesystem::path file_name( file, boost::filesystem::native );
    if ( !file_name.has_root_path() )
    {
      std::string dir_name;
      if ( options_->find( "-n", dir_name ) && !dir_name.empty() )
      {
        create_sub_directory( dir_name );
        boost::filesystem::path dir( dir_name, boost::filesystem::native );
        file_name = dir/file_name;
      }
    }
    return file_name.native_file_string();
  }

  /*!
   *  \brief  TufBNg̍쐬
   *  \param  dir   TufBNg̃pX
   */
  void factory::create_sub_directory( const std::string& dir )
  {
    boost::filesystem::path path( dir, boost::filesystem::native );
    if ( exists( path ) )
    {
      if ( !is_directory( path ) )
      {
        rout.error( _( "%1% is not directory" ) % path.string() );
      }
    }
    else
    {
      try
      {
        create_directory( path );
      }
      catch ( boost::filesystem::filesystem_error& )
      {
        rout.error( _( "cannot make directory `%1%\'" ) % path.string() );
      }
    }
  }

  /*!
   *  \fn     const boost::shared_ptr<kernel_id> factory::do_create_kernel_id( const std::string& file ) const
   *  \brief  create_kernel_id Ăяoۂ̏
   *  \return kernel_id ̃CX^XwX}[g|C^Ԃ
   */

  /*!
   *  \fn     const boost::shared_ptr<kernel_cfg> factory::do_create_kernel_cfg( const std::string& file, const std::string& id_file ) const
   *  \brief  create_kernel_cfg Ăяoۂ̏
   *  \return kernel_cfg ̃CX^XwX}[g|C^Ԃ
   */

  /*!
   *  \fn     const boost::shared_ptr<std::vector<kernel_cfg_generator_type> > factory::do_create_kernel_cfg_generators() const
   *  \brief  create_kernel_cfg_generators Ăяoۂ̏
   *  \return kernel_cfg.cp̐֐QwX}[g|C^Ԃ
   */

  /*!
   *  \fn     const boost::shared_ptr<std::vector<kernel_id_generator_type> > factory::do_create_kernel_id_generators() const
   *  \brief  create_kernel_id_generators Ăяoۂ̏
   *  \return kernel_id.hp̐֐QwX}[g|C^Ԃ
   */

}

