/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2004 by Witz Corporation, JAPAN  
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*!
 *  \file   toppers/factory.hpp
 *  \brief  IuWFNgt@NgɊւ錾`
 *
 *  ̃t@CŒ`NX
 *  \code
 *  class factory;
 *  \endcode
 */
#ifndef TOPPERS_FACTORY_HPP_
#define TOPPERS_FACTORY_HPP_

#include "toppers/cmdline.hpp"
#include <vector>
#include <iosfwd>

namespace toppers
{

  class kernel_id;
  class kernel_cfg;

  /*!
    *  \class  factory factory.hpp "toppers/factory.hpp"
    *  \brief  eIuWFNg̃t@NgNX
    */
  class factory
  {
  public:
    /*!
     *  \brief  kernel_cfg IuWFNg̐֐ւ̃|C^^
     */
    typedef void ( *kernel_cfg_generator_type )( const kernel_cfg&, std::ostream& );
    /*!
     *  \brief  kernel_id IuWFNg̐֐ւ̃|C^^
     */
    typedef void ( *kernel_id_generator_type )( const kernel_id&, std::ostream& );

    factory( const cmdline::option_array& options, const std::string& version );
    /*!
     *  \brief  fXgN^
     */
    virtual ~factory() {}
    const boost::shared_ptr<kernel_id> create_kernel_id() const;
    const boost::shared_ptr<kernel_cfg> create_kernel_cfg() const;
    const boost::shared_ptr<std::vector<kernel_cfg_generator_type> > create_kernel_cfg_generators() const;
    const boost::shared_ptr<std::vector<kernel_id_generator_type> > create_kernel_id_generators() const;
    const boost::shared_ptr<cmdline::option_array>& options() const;
    const std::string& version() const;
  protected:
    const std::string get_cfg_file_name( const std::string& file ) const { return do_get_cfg_file_name( file ); }
    const std::string get_cfg_file_name( const std::string& option, const std::string& default_name ) const;
    virtual const boost::shared_ptr<kernel_id> do_create_kernel_id( const std::string& file ) const = 0;
    virtual const boost::shared_ptr<kernel_cfg> do_create_kernel_cfg( const std::string& file, const std::string& id_file ) const = 0;
    virtual const boost::shared_ptr<std::vector<kernel_cfg_generator_type> > do_create_kernel_cfg_generators() const = 0;
    virtual const boost::shared_ptr<std::vector<kernel_id_generator_type> > do_create_kernel_id_generators() const = 0;
    virtual const std::string do_get_cfg_file_name( const std::string& file ) const;
    static void create_sub_directory( const std::string& dir );
  private:
    boost::shared_ptr<cmdline::option_array> options_;
    std::string version_;
  };

}

#endif  // ! TOPPERS_FACTORY_HPP_
