/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  toppers/itron/cfg_factory.cpp
 */
#include "toppers/itron/cfg_factory.hpp"
#include "toppers/itron/cfg_kernel_id.hpp"
#include "toppers/itron/cfg_kernel_cfg.hpp"
#include "toppers/itron/static_api.hpp"
#include "toppers/itron/cfg_contents.hpp"
#include "toppers/cmdline.hpp"
#include "toppers/itron/specified_factory.hpp"
#include <cassert>

namespace toppers
{
  namespace itron
  {

    /*!
     *  \brief  RXgN^
     *  \param  options   R}hCIvV
     *  \param  version   J[l̃o[Wi"JSP"j
     *  \param  prid      TKERNEL_PRID }N̒l
     */
    cfg_factory::cfg_factory( const cmdline::option_array& options, const std::string& version, long prid )
      : factory( options, version ), prid_( prid )
    {
    }

    const boost::shared_ptr<cfg_contents> cfg_factory::create_cfg_contents( text::const_iterator& next, text::const_iterator last, bool ucn, codeset_t codeset ) const
    {
      return do_create_cfg_contents( next, last, ucn, ascii );
    }

    /*!
     *  \brief  kernel_object ̐
     *  \param  api       ÓIAPI
     *  \param  container  kernel_object ̊i[
     *  \return kernel_object ւ̃X}[g|C^
     */
    const boost::shared_ptr<kernel_object> cfg_factory::create_kernel_object( const static_api& api, std::vector<boost::shared_ptr<kernel_object> >& container )
    {
      boost::shared_ptr<std::vector<kernel_object::builder_type> > builders( do_get_kernel_object_builders() );
      std::vector<boost::shared_ptr<kernel_object> > t( container );
      boost::shared_ptr<kernel_object> ptr;
      typedef std::vector<kernel_object::builder_type>::const_iterator const_iterator;

      for ( const_iterator iter( builders->begin() ), last( builders->end() ); iter != last; ++iter )
      {
        ptr = ( **iter )( api, t );
        if ( !!ptr )
        {
          t.swap( container );
          return ptr;
        }
      }
      if ( !options()->find( "-iapi" ) )
      {
        error( api.line().file_.native_file_string(), api.line().line_, _( "static API `%1%\' cannot be unresolved" ) % api[0] );
      }
      return ptr;
    }

    /*!
     *  \brief  t@Nĝ̐
     *  \param  options   R}hCIvV
     *  \param  version   o[Wi"JSP"j
     *
     *  cfg_factory NXSingletonƂ킯ł͂Ȃ߁AʂɃRXgN^Ă
     *  oĐ邱Ƃł܂A build ֐gp邱ƂŁA register_builder
     *  ֐pēo^֐̒AR}hCIvVo[W
     *  ɓKNXIĐ邱Ƃł悤ɂȂ܂B
     */
    const boost::shared_ptr<cfg_factory> cfg_factory::build( const cmdline::option_array& options, const std::string& version )
    {
      typedef std::vector<builder_type>::const_iterator const_iterator;
      for ( const_iterator iter( builders().begin() ), last( builders().end() ); iter != last; ++iter )
      {
        boost::shared_ptr<cfg_factory> ptr( ( *iter )( options, version ) );
        if ( ptr )
        {
          return ptr;
        }
      }
      return boost::shared_ptr<cfg_factory>();
    }

    /*!
     *  \brief  t@Nĝ̐֐̓o^
     *  \param  builder   ֐
     *
     *  ̊֐œo^֐ build ֐Ăяo܂B
     */
    void cfg_factory::register_builder( builder_type builder )
    {
      if ( builder != 0 )
      {
        builders().push_back( builder );
      }
    }

    /*!
     *  \brief  o^Ăt@Ng֐̏
     */
    void cfg_factory::clear_builders()
    {
      builders().clear();
    }

    const boost::shared_ptr<cfg_contents> cfg_factory::do_create_cfg_contents( text::const_iterator& next, text::const_iterator last, bool ucn, codeset_t codeset ) const
    {
      boost::shared_ptr<cfg_contents> ptr( new cfg_contents( ucn, codeset ) );
      if ( !ptr->parse( next, last ) || next != last )
      {
        ptr.reset();
      }
      return ptr;
    }

    const boost::shared_ptr<kernel_id> cfg_factory::do_create_kernel_id( const std::string& file ) const
    {
      return boost::shared_ptr<kernel_id>( new cfg_kernel_id( file ) );
    }

    const boost::shared_ptr<kernel_cfg> cfg_factory::do_create_kernel_cfg( const std::string& file, const std::string& id_file ) const
    {
      return boost::shared_ptr<kernel_cfg>( new cfg_kernel_cfg( file, id_file, prid() ) );
    }

    /*!
     *  \brief  O[ot@Ng̐ݒ
     *  \param  factory   O[oo^t@Ngւ̃X}[g|C^
     */
    void cfg_factory::global( const boost::shared_ptr<cfg_factory>& factory )
    {
      global_factory() = factory;
    }

    /*!
     *  \brief  O[ot@Ng̎Q
     *  \return O[ot@NgwX}[g|C^Ԃ
     */
    const boost::shared_ptr<cfg_factory> cfg_factory::global()
    {
      return global_factory();
    }

    /*!
     *  \brief  t@Nĝ̃r_֐Rei̎Q
     *  \return r_֐Reiւ̎QƂԂ
     *
     *  ̊֐́AÓIIuWFNg̃RXgN^ĂяoɊւ
     *  邽߂ɓIɎgp܂B
     */
    std::vector<cfg_factory::builder_type>& cfg_factory::builders()
    {
      static std::vector<builder_type> builders_array;
      return builders_array;
    }

    /*!
     *  \brief  O[o cfg_factory IuWFNg̎擾
     *  \return O[o cfg_factory IuWFNgwX}[g|C^ւ̎Q
     *
     *  ̊֐́AÓIIuWFNg̃RXgN^ĂяoɊւ
     *  邽߂ɓIɎgp܂B
     */
    boost::shared_ptr<cfg_factory>& cfg_factory::global_factory()
    {
      static boost::shared_ptr<cfg_factory> instance( build( cmdline::option_array(), "" ) );
      return instance;
    }

    /*!
     *  \fn     const boost::shared_ptr<std::vector<kernel_object::builder_type> > cfg_factory::do_get_kernel_object_builders() const
     *  \brief  create_kernel_object ֐Ŏgp鐶֐Q̎擾
     *  \return ֐Q̃xN^[ւ̃X}[g|C^Ԃ
     */

  }
}
