/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  toppers/itron/cfg_kernel_cfg.cpp
 */
#include "toppers/itron/cfg_kernel_cfg.hpp"
#include "toppers/itron/cfg_factory.hpp"
#include "toppers/itron/cfg_contents.hpp"
#include <ostream>
#include <boost/format.hpp>

namespace toppers
{
  namespace itron
  {

    /*!
     *  \brief  RXgN^
     *  \param  file    kernel_cfg.c i܂͂ɑt@Cj̃t@C
     *  \param  id_file kernel_id.h i܂͂ɑt@Cj̃t@C
     *  \param  prid    TKERNEL_PRID }N̒l
     */
    cfg_kernel_cfg::cfg_kernel_cfg( const std::string& file, const std::string& id_file, long prid )
      : kernel_cfg( file, id_file ), prid_( prid )
    {
    }

    void cfg_kernel_cfg::do_load( const boost::shared_ptr<cfg_contents>& contents )
    {
      typedef cfg_contents::group_container::const_iterator group_iterator;
      const cfg_contents::group_container& container = contents->groups();

      for ( group_iterator giter( contents->groups().begin() ), glast( contents->groups().end() ); giter != glast; ++giter )
      {
        typedef cfg_group::object_container::const_iterator object_iterator;
        const boost::shared_ptr<cfg_group> group( *giter );
        for ( object_iterator oiter( group->objects().begin() ), olast( group->objects().end() ); oiter != olast; ++oiter )
        {
          insert_inib( ( *oiter )->get_object_type(), ( *oiter )->get_inib() );
        }
      }
    }

    void cfg_kernel_cfg::do_body( std::ostream& ostr ) const
    {
      boost::shared_ptr<cmdline::option_array> options( cfg_factory::global()->options() );
      std::string lang;
      bool c_plus_plus = false;
      if ( options->find( "-x", lang ) && lang == "c++" )
      {
        c_plus_plus = true;
      }

      if ( c_plus_plus )
      {
        ostr << "extern \"C\" {\n"
                "\n";
      }

      do_include_headers( ostr );
      do_define_macros( ostr );
      do_check_version( ostr );
      do_define_objects( ostr );
      do_initialize_object( ostr );
      do_initialize_other( ostr );
      do_check_magic_number( ostr );

      if ( c_plus_plus )
      {
        ostr << "}\t// extern \"C\"\n"
                "\n";
      }
    }

    void cfg_kernel_cfg::do_depend() const
    {
    }

    void cfg_kernel_cfg::do_include_headers( std::ostream& ostr ) const
    {
      boost::shared_ptr<cmdline::option_array> options( cfg_factory::global()->options() );
      if ( options->find( "-il" ) )
      {
        ostr << "#include <kernel_cfg.h>\n";
      }
      else
      {
        ostr << "#include \"kernel_cfg.h\"\n";
      }
      ostr << boost::format( "#include \"%1%\"\n" ) % id_file();
      ostr << '\n';
    }

    void cfg_kernel_cfg::do_define_macros( std::ostream& ostr ) const
    {
      ostr << "#define CFG_INTHDR_ENTRY(inthdr) INTHDR_ENTRY(inthdr)\n";
      ostr << "#define CFG_EXCHDR_ENTRY(exchdr) EXCHDR_ENTRY(exchdr)\n";
      ostr << "#define CFG_INT_ENTRY(inthdr) INT_ENTRY(inthdr)\n";
      ostr << "#define CFG_EXC_ENTRY(exchdr) EXC_ENTRY(exchdr)\n";
      ostr << '\n';

      boost::shared_ptr<cmdline::option_array> options( cfg_factory::global()->options() );
      if ( !options->find( "-z" ) )
      {
        ostr << "#ifndef __EMPTY_LABEL\n";
        ostr << "#define __EMPTY_LABEL(x,y) x y[0]\n";
        ostr << "#endif\n";
        ostr << '\n';
      }
      if ( options->find( "-c" ) )
      {
        ostr << "#define __CFG_ASSERT(expr) __CFG_ASSERT2(expr, __LINE__)\n";
        ostr << "#define __CFG_ASSERT2(expr, line) extern void __cfg_assert##line(int __cfg_assertion_failed[(expr)?1:-1])\n";
        ostr << '\n';
      }
    }

    void cfg_kernel_cfg::do_define_objects( std::ostream& ostr ) const
    {
      typedef factory::kernel_cfg_generator_type generator_type;
      const boost::shared_ptr<std::vector<generator_type> > generators( cfg_factory::global()->create_kernel_cfg_generators() );
      std::vector<generator_type>::const_iterator iter;
      const std::vector<generator_type>::const_iterator last( generators->end() );

      for ( iter = generators->begin(); iter != last; ++iter )
      {
        ( *iter )( *this, ostr );
      }
    }

    void cfg_kernel_cfg::do_initialize_object( std::ostream& ostr ) const
    {
      ostr << "void\n";
      ostr << "_kernel_object_initialize(void)\n";
      ostr << "{\n";

      const boost::shared_ptr<std::vector<std::string> > obj_names( cfg_factory::global()->get_kernel_object_names() );

      typedef std::vector<std::string>::const_iterator const_iterator;
      for ( const_iterator iter( obj_names->begin() ), last( obj_names->end() ); iter != last; ++iter )
      {
        const inib_array* pinibs = get_inib_array( *iter );
        if ( pinibs != 0 && !pinibs->empty() )
        {
          // 荞݂CPUOΏۂƂ邽߁AID0̏ꍇLƂ݂ȂB
          if ( pinibs->front()->id() >= 0 )
          {
    	      ostr << "\t_kernel_" << *iter << "_initialize();\n";
          }
        }
      }
      ostr << "}\n";
      ostr << '\n';
    }

    void cfg_kernel_cfg::do_initialize_other( std::ostream& ostr ) const
    {
      cfg_factory::extra_initializer_type einit = cfg_factory::global()->get_extra_initializer();
      ( *einit )( *this, ostr );
    }

    void cfg_kernel_cfg::do_check_version( std::ostream& ostr ) const
    {
      if ( prid_ >= 0 )
      {
        ostr << boost::format( "#if TKERNEL_PRID != %#04xu\n" ) % prid_;
        ostr << "#error \"cannot use this configuration file\"\n";
        ostr << "#endif\n";
        ostr << '\n';
      }
    }

    void cfg_kernel_cfg::do_check_magic_number( std::ostream& ostr ) const
    {
      boost::shared_ptr<cmdline::option_array> options( cfg_factory::global()->options() );
      if ( options->find( "-c" ) )
      {
#if 0 // RtBM[^Ƃ̌݊̂߂ɏo͂Ă_checker_magic_numberp~
        ostr << "\t/* Variables for kernel checker */\n";
        ostr << "const UW _checker_magic_number = 0x01234567;\n";
#endif
      }
    }

  }
}
