/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*!
 *  \file   toppers/itron/chk_factory.hpp
 *  \brief  `FbJ[pt@NgɊւ錾`
 */
#ifndef TOPPERS_ITRON_CHK_FACTORY_HPP_
#define TOPPERS_ITRON_CHK_FACTORY_HPP_

#include "toppers/itron/kernel_chk.hpp"
#include "toppers/cmdline.hpp"
#include <string>
#include <vector>
#include <boost/shared_ptr.hpp>
#include <boost/mpl/for_each.hpp>

namespace toppers
{

  class s_record;
  class nm_symbol;

  namespace itron
  {

    /*!
     *  \class  chk_factory chk_factory.hpp "toppers/itron/chk_factory.hpp"
     *  \brief  `FbJ[pt@NgNX
     */
    class chk_factory
    {
    public:
      /*!
       *  \brief  t@N^֐ւ̃|C^^
       */
      typedef const boost::shared_ptr<chk_factory> ( *builder_type )( const cmdline::option_array&, long );
      /*!
       *  \brief  Ñp[^`FbN֐ւ̃|C^^
       */
      typedef bool ( *checker_type )( const itron::kernel_chk& chk, const s_record& srec, const nm_symbol& syms );

      chk_factory( const cmdline::option_array& options, long prid );
      /*!
       *  \brief  fXgN^
       */
      virtual ~chk_factory() {}

      /*!
       *  \brief  `FbJ[NX̐
       *  \return kernel_chk NXւ̃X}[g|C^Ԃ
       */
      const boost::shared_ptr<kernel_chk> create_kernel_chk() const { return do_create_kernel_chk(); }
      /*!
       *  \brief  Ñp[^`FbN֐Q̎擾
       *  \return `FbN֐ւ̃xN^wX}[g|C^Ԃ
       */
      const boost::shared_ptr<std::vector<checker_type> > get_checkers() const { return do_get_checkers(); }
      /*!
       *  \brief  R}hCIvV̎擾
       *  \return t@NgɎw肵R}hCIvVԂ
       */
      const boost::shared_ptr<cmdline::option_array> options() const { return options_; }
      /*!
       *  \brief  v_NgIDl̎擾
       *  \return t@NgɎw肵v_NgIDlԂ
       */
      long prid() const { return prid_; }

      static const boost::shared_ptr<chk_factory> build( const cmdline::option_array& options, long prid );
      static void register_builder( builder_type builder );
      static void clear_builders();
      static void global( const boost::shared_ptr<chk_factory>& factory );
      static const boost::shared_ptr<chk_factory> global();
    protected:
      virtual const boost::shared_ptr<kernel_chk> do_create_kernel_chk() const = 0;
      virtual const boost::shared_ptr<std::vector<checker_type> > do_get_checkers() const = 0;
    private:
      static std::vector<builder_type>& builders();
      static boost::shared_ptr<chk_factory>& global_factory();

      boost::shared_ptr<cmdline::option_array> options_;
      long prid_;
    };

    /*!
     *  \class  specified_chk_factory chk_factory.hpp "toppers/itron/chk_factory.hpp"
     *  \brief  J[l`FbJpt@Ng𐶐邽߂̃ev[gNX
     *
     */
    template <class Policy>
    class specified_chk_factory : public chk_factory
    {
    public:
      /*!
       *  \brief  RXgN^
       *  \param  options   R}hCIvV
       *  \param  prid      TKERNEL_PRID }N̒l
       */
      specified_chk_factory( const cmdline::option_array& options, long prid )
        : chk_factory( options, prid )
      {
      }

      /*!
       *  \brief  t@Ng̍\z
       *  \param  options   R}hCIvV
       *  \param  prid      t@NgɎw肵v_NgIDl
       *  \return \zɐ΃t@Ngւ̃X}[g|C^Ԃ
       */
      static const boost::shared_ptr<chk_factory> build( const cmdline::option_array& options, long prid )
      {
        boost::shared_ptr<chk_factory> ptr;
        if ( Policy::prid_ == prid )
        {
          ptr.reset( new specified_chk_factory<Policy>( options, prid ) );
        }
        return ptr;
      }
    protected:
      virtual const boost::shared_ptr<kernel_chk> do_create_kernel_chk() const
      {
        return boost::shared_ptr<kernel_chk>( new typename Policy::kernel_chk( "" ) );
      }
      virtual const boost::shared_ptr<std::vector<checker_type> > do_get_checkers() const
      {
        using namespace boost;
        pred_checker pred;
        mpl::for_each< typename Policy::sequence >( pred );
        return pred.result_;
      }
    private:
      struct pred_checker
      {
        typedef checker_type result_type;
        pred_checker() : result_( new std::vector<result_type> ) {}
        template <typename T> void operator()( const T& ) const
        {
          result_->push_back( &T::check );
        }
        boost::shared_ptr<std::vector<result_type> > result_;
      };
    };

  }
}

#endif  // ! TOPPERS_ITRON_CHK_FACTORY_HPP_
