/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  toppers/itron/fdmp/cfg_contents.cpp
 */
#include "toppers/itron/fdmp/cfg_contents.hpp"
#include "toppers/itron/fdmp/class_id.hpp"
#include "toppers/itron/static_api.hpp"
#include "toppers/itron/include.hpp"
#include "toppers/parser.hpp"
#include "toppers/text.hpp"
#include "toppers/reporter.hpp"
#include <map>
#include <algorithm>
#include <iterator>
#include <sstream>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/fstream.hpp>
#include <boost/filesystem/exception.hpp>

namespace toppers
{
  namespace itron
  {
    namespace fdmp
    {

      /*!
       *  }`vZbTJ[lɑ΂ do_classify ֐ł́A[JNXi cfg_group
       *  ƂĊǗjNXƂɎdANXIDԍt܂B
       *  ̌ANXƂɏo͐fBNg쐬A"class_id.h"𐶐܂B
       */
      const boost::shared_ptr<std::vector<boost::shared_ptr<itron::cfg_contents> > > cfg_contents::do_classify( bool ucn, codeset_t codeset ) const
      {
        typedef std::vector<boost::shared_ptr<itron::cfg_contents> > vector_t;

        // ł̓NX[JNXƂď
        // ̃NXɂĂ͎dl̂߁AɑΉ͍sȂB
        typedef std::map<std::string, cfg_group::object_container > map_t;

        boost::shared_ptr<vector_t> v( new vector_t );
        map_t m;
        class_id cls_id;

        // U`ꂽNX}[W
        // ) ̎_ł͊Ƀ}[WĂ͂ł邪Ô
        for ( group_container::const_iterator iter( groups().begin() ), last( groups().end() ); iter != last; ++iter )
        {
          boost::shared_ptr<cfg_group> group( *iter );
          std::string class_name( group->name() );
          std::copy( group->objects().begin(), group->objects().end(), std::back_inserter( m[class_name] ) );
        }

        // NXOIuWFNgQ̎o
        map_t::iterator miter( m.find( "" ) );
        cfg_group::object_container globals;
        if ( miter != m.end() )
        {
          globals = miter->second;
          m.erase( miter );
        }

        // IDԍ̊t
        for ( map_t::iterator iter( m.begin() ), last( m.end() ); iter != last; ++iter )
        {
          // NXOIuWFNg̃}[W
          std::copy( globals.begin(), globals.end(), std::back_inserter( iter->second ) ); 

          if ( !kernel_object::assign_id( iter->second.begin(), iter->second.end() ) )
          {
            v.reset();
            return v;
          }
        }

        // "class_id.h"̏o
        for ( map_t::const_iterator iter( m.begin() ), last( m.end() ); iter != last; ++iter )
        {
          group_container groups;
          boost::shared_ptr<cfg_group> g( new cfg_group );

          g->objects( iter->second );
          groups.push_back( g );
          boost::shared_ptr<itron::cfg_contents> contents( new cfg_contents( iter->first, ucn, codeset ) );
          contents->groups( groups );
          v->push_back( boost::shared_ptr<itron::cfg_contents>( contents ) );

          std::string class_name( iter->first );
          try
          {
            boost::filesystem::path dir( class_name, boost::filesystem::native );
            boost::filesystem::create_directory( dir );
          }
          catch ( boost::filesystem::filesystem_error& )
          {
            toppers::fatal( _( "cannot make directory `%1%\'" ) % class_name );
          }
          cls_id.load( contents );
        }
        cls_id.save();
        return v;
      }

      bool cfg_contents::do_parse( text::const_iterator& next, text::const_iterator last, bool ucn, codeset_t codeset )
      {
        text::const_iterator iter( next );
        while ( iter != last )
        {
          using namespace boost::spirit;
          parse_info<text::const_iterator> info;
          std::string kernel_class;
          std::string class_name;

          // NXO̐ÓIAPIт̉́i_ł"INCLUDE"̂ݑΉj
          boost::shared_ptr<cfg_group> g1( new cfg_group );
          text::const_iterator i( iter );
          static_api api;
          if ( api.parse( i, last, ucn, codeset ) )
          {
            boost::shared_ptr<kernel_object> obj( include::build( api, g1->objects() ) );
            if ( !!obj )
            {
              groups().push_back( g1 );
              iter = i;
            }
          }
          if ( iter == last )
          {
            break;
          }

          // u local_class NX { v̉
          info = boost::spirit::parse( iter, last,
                      lexeme_d[
                        c_ident_p[ assign_a( kernel_class ) ] >> space_p >>
                        c_ident_p[ assign_a( class_name ) ]
                      ] >> '{',
                      space_p
                    );
          if ( !info.hit )
          {
            continue;
//            error( get_text_line( info.stop ), _( "syntax error" ) );
//            return false;
          }
          iter = info.stop;

          // NX̐ÓIAPIт̉
          boost::shared_ptr<cfg_group> g2;
          bool new_group = true;
          for ( group_container::const_iterator iter2( groups().begin() ), last2( groups().end() ); iter2 != last2; ++iter2 )
          {
            if ( ( *iter2 )->name() == class_name && ( *iter2 )->group_type() == kernel_class )
            {
              g2 = *iter2;
              new_group = false;
              break;
            }
          }
          if ( new_group )
          {
            g2.reset( new cfg_group( class_name, kernel_class ) );
          }
          if ( !g2->parse( iter, last, ucn, codeset ) )
          {
            return false;
          }
          
          // u } v̉
          info = boost::spirit::parse( iter, last, ch_p( '}' ), space_p );
          if ( !info.hit )
          {
            error( get_text_line( info.stop ), _( "missing `}\'" ) );
            return false;
          }
          iter = info.stop;

          if ( new_group )
          {
            groups().push_back( g2 );
          }
        }
        next = iter;
        return true;
      }

    }
  }
}
