/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  toppers/itron/fdmp/cfg_factory.cpp
 */
#include "toppers/itron/fdmp/cfg_factory.hpp"
#include "toppers/itron/fdmp/cfg_contents.hpp"
#include <ostream>
#include <cstring>
#include <boost/filesystem/path.hpp>

namespace toppers
{
  namespace itron
  {
    namespace fdmp
    {

    /*!
     *  \brief  RXgN^
     *  \param  options   R}hCIvV
     *  \param  version   J[l̃o[Wi"JSP"j
     *  \param  prid      TKERNEL_PRID }N̒l
     */
      cfg_factory::cfg_factory( const cmdline::option_array& options, const std::string& version, long prid )
        : itron::cfg_factory( options, version, prid )
      {
      }

    /*!
     *  \brief  t@Nĝ̐
     *  \param  options   R}hCIvV
     *  \param  version   o[Wi"FDMP"j
     *
     *  cfg_factory NXSingletonƂ킯ł͂Ȃ߁AʂɃRXgN^Ă
     *  oĐ邱Ƃł܂A build ֐gp邱ƂŁA register_builder
     *  ֐pēo^֐̒AR}hCIvVo[W
     *  ɓKNXIĐ邱Ƃł悤ɂȂ܂B
     */
      const boost::shared_ptr<itron::cfg_factory> cfg_factory::build( const cmdline::option_array& options, const std::string& version )
      {
        boost::shared_ptr<itron::cfg_factory> ptr;
        const char* ver = version.c_str();
        if ( std::strncmp( ver, "FDMP", 2 ) == 0 || std::strncmp( ver, "fdmp", 2 ) == 0 )
        {
          if ( ver[2] == '-' )
          {
            ver = ver + 3;
          }
          else
          {
            ver = "";
          }
          boost::shared_ptr<itron::cfg_factory> base( itron::cfg_factory::build( options, ver ) );
          if ( !!base )
          {
            ptr.reset( new cfg_factory( options, version, -1L ) );
            boost::static_pointer_cast<cfg_factory>( ptr )->base_factory_ = base;
          }    
        }
        return ptr;
      }

      const boost::shared_ptr<itron::cfg_contents> cfg_factory::do_create_cfg_contents( text::const_iterator& next, text::const_iterator last, bool ucn, codeset_t codeset ) const
      {
        boost::shared_ptr<itron::cfg_contents> ptr( new fdmp::cfg_contents( ucn, codeset ) );
        if ( !ptr->parse( next, last ) || next != last )
        {
          ptr.reset();
        }
        return ptr;
      }

      const std::string cfg_factory::do_get_cfg_file_name( const std::string& file ) const
      {
        boost::filesystem::path path( file, boost::filesystem::native );
        boost::shared_ptr<itron::cfg_contents> contents( get_contents() );
        if ( !!contents && !contents->name().empty() )
        {
          boost::filesystem::path dir( contents->name(), boost::filesystem::native );
          path = dir/path;
        }
        return itron::cfg_factory::do_get_cfg_file_name( path.native_file_string() );
      }

      const boost::shared_ptr<std::vector<factory::kernel_cfg_generator_type> > cfg_factory::do_create_kernel_cfg_generators() const
      {
        return base_factory_->create_kernel_cfg_generators();
      }

      const boost::shared_ptr<std::vector<factory::kernel_id_generator_type> > cfg_factory::do_create_kernel_id_generators() const
      {
        return base_factory_->create_kernel_id_generators();
      }

      const boost::shared_ptr<std::vector<kernel_object::builder_type> > cfg_factory::do_get_kernel_object_builders() const
      {
        return base_factory_->get_kernel_object_builders();
      }

      const boost::shared_ptr<std::vector<std::string> > cfg_factory::do_get_kernel_object_names() const
      {
        return base_factory_->get_kernel_object_names();
      }

      itron::cfg_factory::extra_initializer_type cfg_factory::do_get_extra_initializer() const
      {
        return &extra_init;
      }

      /*!
       *  \brief  gR[h̐
       *  \param  cfg   kernel_cfg.c 
       *  \param  ostr  o̓Xg[
       *
       *  ̊֐̓}`vZbTJ[lɕKv init_gccb_object ֐𐶐܂B
       *  ̑AJ[l̎dlύXɉđ̏KvɂȂꍇÅ֐
       *  ύXĂB
       */
      void cfg_factory::extra_init( const kernel_cfg& cfg, std::ostream& ostr )
      {
		  
		/* @chg 2004-04-03 init_gccb_object() ̔p~ɂύX */  
        const boost::shared_ptr<itron::cfg_factory> base_factory( boost::dynamic_pointer_cast<cfg_factory>( global() )->base_factory_ );
#if 0		  
        ostr << "void\n"
                "init_gccb_object(void)\n"
                "{\n";

        const boost::shared_ptr<std::vector<std::string> > obj_names( base_factory->get_kernel_object_names() );

        typedef std::vector<std::string>::const_iterator const_iterator;
        for ( const_iterator iter( obj_names->begin() ), last( obj_names->end() ); iter != last; ++iter )
        {
          const kernel_cfg::inib_array* pinibs = cfg.get_inib_array( *iter );
          if ( pinibs != 0 && !pinibs->empty() )
          {
            // 荞݂CPUOΏۂƂ邽߁AID0̏ꍇLƂ݂ȂB
            if ( pinibs->front()->id() >= 0 )
            {
    	        ostr << "\tinit_gccb_" << *iter << "();\n";
            }
          }
        }

        ostr << "}\n"
                "\n";
#endif /* 0 */	  				  
        extra_initializer_type einit = base_factory->get_extra_initializer();
        ( *einit )( cfg, ostr );
      }

      const boost::shared_ptr<kernel_chk> cfg_factory::do_create_kernel_chk() const
      {
        return base_factory_->create_kernel_chk();
      }
    }
  }
}
