/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  toppers/itron/fdmp/class_id.cpp
 */
#include "toppers/itron/fdmp/class_id.hpp"
#include "toppers/itron/cfg_factory.hpp"
#include "toppers/itron/cfg_contents.hpp"
#include "toppers/reporter.hpp"
#include <sstream>
#include <fstream>
#include <boost/format.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/fstream.hpp>
#include <boost/filesystem/exception.hpp>

namespace toppers
{
  namespace itron
  {
    namespace fdmp
    {

      /*!
       *  \brief  RXgN^
       *  \param  file  t@Ciʏ"class_id.h"j
       */
      class_id::class_id( const std::string& file ) : kernel_id( file )
      {
      }

      void class_id::load( const boost::shared_ptr<itron::cfg_contents>& contents )
      {
        typedef cfg_contents::group_container::const_iterator group_iterator;
        const cfg_contents::group_container& container = contents->groups();

        for ( group_iterator giter( contents->groups().begin() ), glast( contents->groups().end() ); giter != glast; ++giter )
        {
          typedef cfg_group::object_container::const_iterator object_iterator;
          const boost::shared_ptr<cfg_group> group( *giter );
          for ( object_iterator oiter( group->objects().begin() ), olast( group->objects().end() ); oiter != olast; ++oiter )
          {
            std::string key;
            key = contents->name() + ' ' + ( *oiter )->get_object_type();
            insert_id( key, ( *oiter )->get_idb() );
          }
          classes_.push_back( contents->name() );
        }
      }
   
      void class_id::save() const
      {
        std::ostringstream osstr;
        if ( do_save( osstr ) )
        {
          for ( std::vector<std::string>::const_iterator iter( classes_.begin() ), last( classes_.end() ); iter != last; ++iter )
          {
            using namespace boost::filesystem;
            ofstream ofstr( path( *iter, native )/path( file(), native ) );
            if ( ofstr.fail() )
            {
              error( _( "cannot create file `%1%\'" ) % *iter );
            }
            ofstr << osstr.str() << std::endl;
            ofstr.close();
            do_depend();
          }
        }
      }

      void class_id::do_body( std::ostream& ostr ) const
      {
        std::vector<std::string> object_types( do_get_object_types() );
        typedef std::vector<std::string>::const_iterator const_iterator;
        long id = 0;

        for ( const_iterator citer( classes_.begin() ), clast( classes_.end() ); citer != clast; ++citer )
        {
          ++id; // NXIDԍ͂Ƃ肠PȘA
          ostr << boost::format( "#define %1%_CLASS_ID\t%2%" ) % *citer % id << "\n\n";
          for ( const_iterator titer( object_types.begin() ), tlast( object_types.end() ); titer != tlast; ++titer )
          {
            const kernel_id::id_array* pids = get_id_array( *citer + ' ' + *titer );
            if ( pids != 0 )
            {
              typedef kernel_id::id_array::const_iterator idb_iterator;
              for ( idb_iterator iter( pids->begin() ), last( pids->end() ); iter != last; ++iter )
              {
                std::string name( ( *iter )->name() );
                if ( !name.empty() )
                {
                  ostr << boost::format( "#define %1%_%2%\t((%1%_CLASS_ID << 16) | %3%)" ) % *citer % name % ( *iter )->value() << '\n';
                }
              }
            }
          }
          ostr << '\n';
        }
      }

      const std::vector<std::string>& class_id::do_get_object_types() const
      {
        static const char* object_types[] =
        {
          "task", "semaphore", "eventflag", "dataqueue", "mailbox", "mempfix", "cyclic"
        };
        static std::vector<std::string> object_type_array( object_types, object_types + sizeof( object_types )/sizeof( object_types[0] ) );
        return object_type_array;
      }

    }
  }
}
