/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  toppers/itron/include.cpp
 */
#include "toppers/itron/include.hpp"
#include "toppers/itron/static_api.hpp"

namespace toppers
{
  namespace itron
  {

    /*!
     *  \brief  specified_object<Plociy> NXv Policy::generate_kernel_cfg ֐
     *  \param  cfg   kernel_cfg.c 
     *  \param  ostr  o̓Xg[
     *  \return ɐ true Ԃ
     */
    bool include_policy::generate_kernel_cfg( const kernel_cfg& cfg, std::ostream& ostr )
    {
      using boost::format;
      typedef kernel_cfg::inib_array::const_iterator const_iterator;

      const kernel_cfg::inib_array* pinibs = cfg.get_inib_array( "include" );
      if ( pinibs == 0 )
      {
        static const kernel_cfg::inib_array dummy;
        pinibs = &dummy;
      }

      ostr << "\t/* User specified include files */\n\n";
      for ( const_iterator iter( pinibs->begin() ), last( pinibs->end() ); iter != last; ++iter )
      {
        std::string path( ( *iter )->get( "path" ) );
        ostr << format( "#include %1%\n" ) % path;
      }
      ostr << '\n';

      return true;
    }

    /*!
     *  \brief  kernel_id.h ւ̏o͍͂sȂ
     *  \return  true Ԃ
     */
    bool include_policy::generate_kernel_id( const kernel_id&, std::ostream& )
    {
      return true;
    }

    /*!
     *  \brief  specified_object<Plociy> NXv Policy::verify ֐
     *  \param  api   ͑Ώۂ̐ÓIAPI
     *  \param  obj   J[lIuWFNg
     *  \return 茋ʂł true ԂB
     */
    bool include_policy::verify( const static_api& api, kernel_object& obj )
    {
      std::string path( obj.get( "path" ) );
      std::string::size_type size = path.size();

      if ( path[0] != '\"' || path[size-1] != '\"' )
      {
        static_api::line_type line( api.line() );
        error( line.file_.native_file_string(), line.line_, _( "`INCLUDE\' expects string literal" ) );
        return false;
      }
      path = path.substr( 1, size-2 );  // [ '"' 
      
      // \"  " ɕϊ
      for ( std::string::size_type pos = path.find( "\\\"" ); pos != std::string::npos; pos = path.find( "\\\"", pos+1 ) )
      {
        path.replace( pos, 2, "\"" );
      }

      obj.set( "path", path );
      return true;
    }

    const char include_policy::name_[] = "include";
    const char* const include_policy::apis_[] = { "INCLUDE", 0 };
    const char* const include_policy::params_[] = { "path", 0, };

  }
}
