/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  toppers/itron/jsp/cyclic.cpp
 */
#include "toppers/itron/jsp/cyclic.hpp"
#include "toppers/itron/jsp/kernel_chk.hpp"
#include "toppers/reporter.hpp"
#include "toppers/s_record.hpp"
#include "toppers/nm_symbol.hpp"

namespace toppers
{
  namespace itron
  {
    namespace jsp
    {

      /*!
       *  \brief  p[^`FbNR[h ̐
       *  \param  cfg   kernel_cfg.c 
       *  \param  ostr  o̓Xg[
       */
      void cyclic_policy::generate_assertions( const kernel_cfg& cfg, std::ostream& ostr )
      {
        const kernel_cfg::inib_array* pinibs = cfg.get_inib_array( name_ );
        static const char pattern[] = 
          "#if ((@#cycatr) & (TA_HLNG|TA_ASM)) != TA_HLNG\n"
          "#error \"`cycatr\' of cyclic handler `@name\' is not TA_HLNG\"\n"
          "#endif\n"
          "__CFG_ASSERT(((@#cycatr) & (TA_HLNG|TA_ASM)) == TA_HLNG);\n"
          "#if ((@#cycatr) & 0x04u) != 0x00u\n"
          "#error \"`cycatr\' of cyclic handler `@name\' is specified TA_PHS\"\n"
          "#endif\n"
          "__CFG_ASSERT(((@#cycatr) & 0x04u) == 0x00u);\n"
          "\n";
        std::transform( pinibs->begin(), pinibs->end(), std::ostream_iterator<std::string>( ostr ), generator( pattern ) );
      }

      /*!
       *  \brief  Ñp[^`FbN
       *  \param  chk   `FbN
       *  \param  srec  [hW[̃_v
       *  \param  syms  V{e[u
       */
      bool cyclic_policy::check( const itron::kernel_chk& chk, const s_record& srec, const nm_symbol& syms )
      {
        nm_symbol::entry entry = syms.find( "_kernel_cycinib_table" );
        if ( entry.type < 0 )
        {
          return false;
        }
        long tnum_cycid = chk.get( kernel_chk::countof_CYCID );
        long sizeof_FP = chk.get( kernel_chk::sizeof_FP );
        long sizeof_CYCINIB = chk.get( kernel_chk::sizeof_CYCINIB );
        long offsetof_CYCINIB_cychdr = chk.get( kernel_chk::offsetof_CYCINIB_cychdr );

        for ( int i = 0; i < tnum_cycid; i++ )
        {
          unsigned long value = 0;
          for ( int j = 0; j < sizeof_FP; j++ )
          {
            int t = srec[entry.address + sizeof_CYCINIB*i + offsetof_CYCINIB_cychdr + j];
            if ( t < 0 )
            {
              return false;
            }
            value = ( value << 8 ) | ( t & 0xff );
          }
          if ( value == 0 )
          {
            error( _( "start address of cyclic %1% handler is null" ) % ( i + 1 ) );
          }
        }
        return true;
      }

      const char cyclic_policy::name_[] = "cyclic";
      const char cyclic_policy::symbol_[] = "cyc";
      const char* const cyclic_policy::apis_[] = { "CRE_CYC", 0 };
      const char* const cyclic_policy::params_[] = { "$cycid { #cycatr exinf cychdr cyctim cycphs }", 0 };
      const char cyclic_policy::init_format_[] = "\t{ @#cycatr, @exinf, (FP)(@cychdr), @cyctim, @cycphs }";

    }
  }
}
