/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  toppers/itron/jsp/exception.cpp
 */
#include "toppers/itron/jsp/exception.hpp"
#include "toppers/itron/jsp/kernel_chk.hpp"
#include "toppers/reporter.hpp"
#include "toppers/s_record.hpp"
#include "toppers/nm_symbol.hpp"
#include <boost/lexical_cast.hpp>

namespace toppers
{
  namespace itron
  {
    namespace jsp
    {

      /*!
       *  \brief  p[^`FbNR[h ̐
       *  \param  cfg   kernel_cfg.c 
       *  \param  ostr  o̓Xg[
       */
      void exception_policy::generate_assertions( const kernel_cfg& cfg, std::ostream& ostr )
      {
        const kernel_cfg::inib_array* pinibs = cfg.get_inib_array( name_ );
        static const char pattern[] = 
          "#if (@#excatr) != 0x00u\n"
          "#error \"`excatr\' of CPU exception handler `@name\' is not 0\"\n"
          "#endif\n"
          "__CFG_ASSERT((@#excatr) == 0x00u);\n"
          "\n";
        std::transform( pinibs->begin(), pinibs->end(), std::ostream_iterator<std::string>( ostr ), generator( pattern ) );
      }

      /*!
       *  \brief  specified_object<Plociy> NXv Policy::verify ֐
       *  \param  api   ͑Ώۂ̐ÓIAPI
       *  \param  obj   J[lIuWFNg
       *  \return 茋ʂł true ԂB
       */
      bool exception_policy::verify( const static_api& api, kernel_object& obj )
      {
        try
        {
          if ( boost::lexical_cast<long>( obj.get( "%excno" ) ) < 0 )
          {
            warning( _( "parameter `%1%\' is negative" ) % "excno" );
          }
        }
        catch ( boost::bad_lexical_cast& )
        {
          // excno͕K萔ł͂Ȃ̂ŁAOĂɖ͂Ȃ
        }
        return true;
      }

      /*!
       *  \brief  Ñp[^`FbN
       *  \param  chk   `FbN
       *  \param  srec  [hW[̃_v
       *  \param  syms  V{e[u
       */
      bool exception_policy::check( const itron::kernel_chk& chk, const s_record& srec, const nm_symbol& syms )
      {
        nm_symbol::entry entry = syms.find( "_kernel_excinib_table" );
        if ( entry.type < 0 )
        {
          return false;
        }
        long tnum_cycid = chk.get( kernel_chk::countof_EXCNO );
        long sizeof_FP = chk.get( kernel_chk::sizeof_FP );
        long sizeof_EXCINIB = chk.get( kernel_chk::sizeof_EXCINIB );
        long offsetof_EXCINIB_cychdr = chk.get( kernel_chk::offsetof_EXCINIB_exchdr );

        for ( int i = 0; i < tnum_cycid; i++ )
        {
          unsigned long value = 0;
          for ( int j = 0; j < sizeof_FP; j++ )
          {
            int t = srec[entry.address + sizeof_EXCINIB*i + offsetof_EXCINIB_cychdr + j];
            if ( t < 0 )
            {
              return false;
            }
            value = ( value << 8 ) | ( t & 0xff );
          }
          if ( value == 0 )
          {
            error( _( "start address of CPU exception #%1% handler is null" ) % ( i + 1 ) );
          }
        }
        return true;
      }

      const char exception_policy::name_[] = "exception";
      const char exception_policy::symbol_[] = "exc";
      const char* const exception_policy::apis_[] = { "DEF_EXC", 0 };
      const char* const exception_policy::params_[] = { "%excno { #excatr exchdr }" };
      const char exception_policy::region_format_[] = "CFG_EXCHDR_ENTRY(@exchdr);\n";
      const char exception_policy::init_format_[] = "\t{ @%excno, @#excatr, (FP)CFG_EXC_ENTRY(@exchdr) }";

    }
  }
}
