/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  toppers/itron/jsp/kernel_chk.cpp
 */
#include "toppers/itron/jsp/kernel_chk.hpp"
#include "toppers/itron/jsp/kernel_policy.hpp"
#include "toppers/s_record.hpp"
#include "toppers/reporter.hpp"
#include <ostream>
#include <boost/lexical_cast.hpp>

namespace toppers
{
  namespace itron
  {
    namespace jsp
    {

      void kernel_chk::do_body( std::ostream& ostr ) const
      {
        ostr << "#include \"kernel_cfg.h\"\n"
                "#include <stddef.h>\n"
                "\n";

        ostr << "const UW kernel_chk[] = {\n";

        std::string table[table_size];      

        table[magic_number] = "0x12345678";
        table[TKERNEL_PRID] = boost::lexical_cast<std::string>( (int)kernel_policy::prid_ ) + "\t/* TKERNEL_PRID */";

        table[sizeof_ID] = "sizeof(ID)";
        table[sizeof_UINT] = "sizeof(UINT)";
        table[sizeof_SIZE] = "sizeof(SIZE)";
        table[sizeof_ATR] = "sizeof(ATR)";
        table[sizeof_PRI] = "sizeof(PRI)";
        table[sizeof_RELTIM] = "sizeof(RELTIM)";
        table[sizeof_VP] = "sizeof(VP)";
        table[sizeof_VP_INT] = "sizeof(VP_INT)";
        table[sizeof_FP] = "sizeof(FP)";

        table[countof_TSKID] = boost::lexical_cast<std::string>( countof( "task" ) ) + "\t/* TNUM_TSKID */";
        table[sizeof_TINIB] = "sizeof(TINIB)";
        table[offsetof_TINIB_tskatr] = "offsetof(TINIB, tskatr)";
        table[offsetof_TINIB_exinf] = "offsetof(TINIB, exinf)";
        table[offsetof_TINIB_task] = "offsetof(TINIB, task)";
        table[offsetof_TINIB_ipriority] = "offsetof(TINIB, ipriority)";
        table[offsetof_TINIB_stksz] = "offsetof(TINIB, stksz)";
        table[offsetof_TINIB_stk] = "offsetof(TINIB, stk)";
        table[offsetof_TINIB_texatr] = "offsetof(TINIB, texatr)";
        table[offsetof_TINIB_texrtn] = "offsetof(TINIB, texrtn)";

        table[countof_SEMID] = boost::lexical_cast<std::string>( countof( "semaphore" ) ) + "\t/* TNUM_SEMID */";
        table[sizeof_SEMINIB] = "sizeof(SEMINIB)";
        table[offsetof_SEMINIB_sematr] = "offsetof(SEMINIB, sematr)";
        table[offsetof_SEMINIB_isemcnt] = "offsetof(SEMINIB, isemcnt)"; 
        table[offsetof_SEMINIB_maxsem] = "offsetof(SEMINIB, maxsem)";

        table[countof_FLGID] = boost::lexical_cast<std::string>( countof( "eventflag" ) ) + "\t/* TNUM_FLGID */";
        table[sizeof_FLGPTN] = "sizeof(FLGPTN)";
        table[sizeof_FLGINIB] = "sizeof(FLGINIB)";
        table[offsetof_FLGINIB_flgatr] = "offsetof(FLGINIB, flgatr)";
        table[offsetof_FLGINIB_iflgptn] = "offsetof(FLGINIB, iflgptn)"; 

        table[countof_DTQID] = boost::lexical_cast<std::string>( countof( "dataqueue" ) ) + "\t/* TNUM_DTQID */";
        table[sizeof_DTQINIB] = "sizeof(DTQINIB)";
        table[offsetof_DTQINIB_dtqatr] = "offsetof(DTQINIB, dtqatr)";
        table[offsetof_DTQINIB_dtqcnt] = "offsetof(DTQINIB, dtqcnt)";
        table[offsetof_DTQINIB_dtq] = "offsetof(DTQINIB, dtq)";

        table[countof_MBXID] = boost::lexical_cast<std::string>( countof( "mailbox" ) ) + "\t/* TNUM_MBXID */";
        table[sizeof_MBXINIB] = "sizeof(MBXINIB)";
        table[offsetof_MBXINIB_mbxatr] = "offsetof(MBXINIB, mbxatr)";
        table[offsetof_MBXINIB_maxmpri] = "offsetof(MBXINIB, maxmpri)"; 

        table[countof_MPFID] = boost::lexical_cast<std::string>( countof( "mempfix" ) ) + "\t/* TNUM_MPFID */";
        table[sizeof_MPFINIB] = "sizeof(MPFINIB)";
        table[offsetof_MPFINIB_mpfatr] = "offsetof(MPFINIB, mpfatr)";
        table[offsetof_MPFINIB_blksz] = "offsetof(MPFINIB, blksz)";
        table[offsetof_MPFINIB_mpf] = "offsetof(MPFINIB, mpf)";
        table[offsetof_MPFINIB_limit] = "offsetof(MPFINIB, limit)";

        table[countof_CYCID] = boost::lexical_cast<std::string>( countof( "cyclic" ) ) + "\t/* TNUM_CYCID */";
        table[sizeof_CYCINIB] = "sizeof(CYCINIB)";
        table[offsetof_CYCINIB_cycatr] = "offsetof(CYCINIB, cycatr)";
        table[offsetof_CYCINIB_exinf] = "offsetof(CYCINIB, exinf)";
        table[offsetof_CYCINIB_cychdr] = "offsetof(CYCINIB, cychdr)";
        table[offsetof_CYCINIB_cyctim] = "offsetof(CYCINIB, cyctim)";
        table[offsetof_CYCINIB_cycphs] = "offsetof(CYCINIB, cycphs)";

        table[countof_INHNO] = boost::lexical_cast<std::string>( countof( "interrupt" ) ) + "\t/* TNUM_INHNO */";
        table[sizeof_INHNO] = "sizeof(INHNO)";
        table[sizeof_INHINIB] = "sizeof(INHINIB)";
        table[offsetof_INHINIB_inhno] = "offsetof(INHINIB, inhno)";
        table[offsetof_INHINIB_inhatr] = "offsetof(INHINIB, inhatr)"; 
        table[offsetof_INHINIB_inthdr] = "offsetof(INHINIB, inthdr)";

        table[countof_EXCNO] = boost::lexical_cast<std::string>( countof( "exception" ) ) + "\t/* TNUM_EXCNO */";
        table[sizeof_EXCNO] = "sizeof(EXCNO)";
        table[sizeof_EXCINIB] = "sizeof(EXCINIB)";
        table[offsetof_EXCINIB_excno] = "offsetof(EXCINIB, excno)";
        table[offsetof_EXCINIB_excatr] = "offsetof(EXCINIB, excatr)"; 
        table[offsetof_EXCINIB_exchdr] = "offsetof(EXCINIB, exchdr)";

        for ( int i = 0; i < table_size; i++ )
        {
          ostr << '\t' << table[i] << ",\n";
        }

        ostr << "};\n";
      }

      bool kernel_chk::do_load( const s_record& srec, unsigned long address )
      {
        bool result = true;
        set( table_size-1, 0 ); // ̂߁AI[vfł

        for ( int i = 0; i < table_size; i++ )
        {
          unsigned long value = 0;
          for ( int j = 0; j < 4; j++ )
          {
            value = value << 8 | srec[address + i*4 + j];
          }
          set( i, value );
        }

        switch ( get( magic_number ) )
        {
        case 0x12345678:
          byteorder( big_endian );
          break;
        case 0x78563412:
          byteorder( little_endian );
          for ( int i = 0; i < table_size; i++ )
          {
            unsigned long value = 0;
            unsigned long t = get( i );
            for ( int j = 0; j < 4; j++ )
            {
              value = value << 8 | ( t & 0xff );
              t >>= 8;
            }
            set( i, value );
          }
          break;
        default:
          result = false;
          break;
        }
        return result;
      }

    }
  }
}
