/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  toppers/itron/jsp/task.cpp
 */
#include "toppers/itron/jsp/task.hpp"
#include "toppers/itron/jsp/kernel_chk.hpp"
#include "toppers/reporter.hpp"
#include "toppers/misc.hpp"
#include "toppers/s_record.hpp"
#include "toppers/nm_symbol.hpp"

namespace toppers
{
  namespace itron
  {
    namespace jsp
    {

      namespace
      {
        
        long get_id( boost::shared_ptr<kernel_cfg::initial_block> pinib )
        {
          return pinib->id();
        }

      }

      /*!
       *  \brief  torder_table ̐
       *  \param  cfg   kernel_cfg.c 
       *  \param  ostr  o̓Xg[
       */
      void task_policy::generate_constants( const kernel_cfg& cfg, std::ostream& ostr )
      {
        ostr << "const ID _kernel_torder_table[TNUM_TSKID] = {";
        const kernel_cfg::inib_array* pinibs = cfg.get_inib_array( name_ );
        output_list( pinibs->begin(), pinibs->end(), ostr, get_id, ", " );
        ostr << "};\n\n";
      }

      /*!
       *  \brief  p[^`FbNR[h ̐
       *  \param  cfg   kernel_cfg.c 
       *  \param  ostr  o̓Xg[
       */
      void task_policy::generate_assertions( const kernel_cfg& cfg, std::ostream& ostr )
      {
        const kernel_cfg::inib_array* pinibs = cfg.get_inib_array( name_ );
        static const char pattern[] = 
          "#if ((@#tskatr) & (TA_HLNG|TA_ASM)) != TA_HLNG\n"
          "#error \"`tskatr\' of task `@name\' is not TA_HLNG\"\n"
          "#endif\n"
          "__CFG_ASSERT(((@#tskatr) & (TA_HLNG|TA_ASM)) == TA_HLNG);\n"
          "__CFG_ASSERT(TMIN_TPRI <= (@itskpri) && (@itskpri) <= TMAX_TPRI);\n"
          "__CFG_ASSERT((@stksz) > 0);\n"
          "\n"
          "#if ((@#texatr) & (TA_HLNG|TA_ASM)) != TA_HLNG\n"
          "#error \"`texatr\' of task `@name\' is not TA_HLNG\"\n"
          "#endif\n"
          "__CFG_ASSERT(((@#texatr) & (TA_HLNG|TA_ASM)) == TA_HLNG);\n"
          "\n";
        std::transform( pinibs->begin(), pinibs->end(), std::ostream_iterator<std::string>( ostr ), generator( pattern ) );
      }

      /*!
       *  \brief  ^XNȂꍇ __EMPTY_LABEL ̐
       *  \param  ostr  o̓Xg[
       */
      void task_policy::generate_extra_empty_labels( std::ostream& ostr )
      {
        ostr << "__EMPTY_LABEL(const ID, _kernel_torder_table);\n";
      }

      /*!
       *  \brief  specified_object<Plociy> NXv Policy::verify ֐
       *  \param  api   ͑Ώۂ̐ÓIAPI
       *  \param  obj   J[lIuWFNg
       *  \return 茋ʂł true ԂB
       */
      bool task_policy::verify( const static_api& api, kernel_object& obj )
      {
        if ( obj.get( "stk" ) != "NULL" )
        {
          error( _( "parameter `%1%\' must be `NULL\'" ) % "stk" );
          return false;
        }
        if ( obj.get( "#texatr" ).empty() )
        {
          obj.set( "#texatr", "0" );
        }
        if ( obj.get( "texrtn" ).empty() )
        {
          obj.set( "texrtn", "0" );
        }
        return true;
      }

      /*!
       *  \brief  Ñp[^`FbN
       *  \param  chk   `FbN
       *  \param  srec  [hW[̃_v
       *  \param  syms  V{e[u
       */
      bool task_policy::check( const itron::kernel_chk& chk, const s_record& srec, const nm_symbol& syms )
      {
        nm_symbol::entry entry = syms.find( "_kernel_tinib_table" );
        if ( entry.type < 0 )
        {
          return false;
        }
        long tnum_tskid = chk.get( kernel_chk::countof_TSKID );
        long sizeof_FP = chk.get( kernel_chk::sizeof_FP );
        long sizeof_TINIB = chk.get( kernel_chk::sizeof_TINIB );
        long offsetof_TINIB_task = chk.get( kernel_chk::offsetof_TINIB_task );

        for ( int i = 0; i < tnum_tskid; i++ )
        {
          unsigned long value = 0;
          for ( int j = 0; j < sizeof_FP; j++ )
          {
            int t = srec[entry.address + sizeof_TINIB*i + offsetof_TINIB_task + j];
            if ( t < 0 )
            {
              return false;
            }
            value = ( value << 8 ) | ( t & 0xff );
          }
          if ( value == 0 )
          {
            error( _( "start address of task %1% is null" ) % ( i + 1 ) );
          }
        }
        return true;
      }

      const char task_policy::name_[] = "task";
      const char task_policy::symbol_[] = "tsk";
      const char* const task_policy::apis_[] = { "CRE_TSK", 0 };
      const char* const task_policy::params_[] = { "$tskid { #tskatr exinf task itskpri stksz stk }", 0 };
      const char* const task_policy::apis2_[] = { "DEF_TEX", 0 };
      const char* const task_policy::params2_[] = { "%tskid { #texatr texrtn }", 0 };
      const char* const task_policy::prefix_ = "t";
      const char task_policy::region_format_[] = "static __STK_UNIT __stack_@name[__TCOUNT_STK_UNIT(@stksz)];\n";
      const char task_policy::init_format_[] = "\t{ @#tskatr, (VP_INT)(@exinf), (FP)(@task), INT_PRIORITY(@itskpri), __TROUND_STK_UNIT(@stksz), __stack_@name, @#texatr, (FP)(@texrtn) }";

    }
  }
}
