/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  toppers/itron/kernel_chk.cpp
 */
#include "toppers/itron/kernel_chk.hpp"
#include "toppers/itron/cfg_contents.hpp"
#include "toppers/itron/kernel_object.hpp"
#include "toppers/s_record.hpp"
#include "toppers/reporter.hpp"

namespace toppers
{
  namespace itron
  {

    /*!
     *  \brief  "kernel_chk.srec"̃`FbN̓ǂݍ
     *  \param  srec    "kernel_chk.srec"̓e
     *  \param  address ǂݍ݊JnԒn
     *
     *  address ~0ULiftHgljw肷ƁASR[h̍ŏǂݍ݂܂B
     */
    void kernel_chk::load( const s_record& srec, unsigned long address )
    {
      if ( address == ~0UL )
      {
        address = srec.lower_bound();
      }
      byteorder_ = unknown;

      while ( !do_load( srec, address ) )
      {
        if ( srec.upper_bound() <= address + 1 )
        {
          error( _( "illegal check script" ) );
        }
      }
    }

    /*!
     *  \brief  J[lIuWFNg̎擾
     *  \param  object_type J[lIuWFNg̎
     *  \return w肵ނ̃J[lIuWFNgԂ
     *
     *  "task"̃J[lIuWFNǧ𒲂ׂ܂B
     */
    long kernel_chk::countof( const std::string& object_type ) const
    {
      std::map<std::string, long>::const_iterator iter( countof_map_.find( object_type ) );
      if ( iter != countof_map_.end() )
      {
        return iter->second;
      }
      return 0;
    }

    /*!
     *  \brief  "kernel_chk.c"̔zvf̎Q
     *  \param  key   z̃CfbNX
     *  \return key Ŏw肵zvfiUW^j̒lԂ
     */
    unsigned long kernel_chk::get( int key ) const
    {
      if ( key < 0 || static_cast<int>( table_.size() ) <= key )
      {
        return 0;
      }
      return table_[key];
    }

    /*!
     *  \brief  "kernel_chk.c"̔zvf̐ݒ
     *  \param  key   z̃CfbNX
     *  \param  value zvf̒lɐݒ肷l
     *
     *  ̊֐ kernel_chk IuWFNgIɊǗzݒ肷邽߂Ɏgp܂B
     *  ̊֐ĂяoĂA"kernel_chk.srec""kernel_chk.o", "kernel_chk.c"ɉeo
     *  킯ł͂܂B
     */
    void kernel_chk::set( int key, unsigned long value )
    {
      if ( key >= 0 )
      {
        if ( key >= static_cast<int>( table_.size() ) )
        {
          table_.resize( key + 1 );
        }
        table_[key] = value;
      }
    }

    /*!
     *  \brief  v_NgIDԍ̓ǂݍ
     *  \param  srec    "kernel_chk.srec"t@C̓e
     *  \param  address JnAhX
     *
     *  v_NgIDԍ"kernel_chk.c"ɏo͂QԖڂ̗vfɊi[ĂƑz肵܂B
     *  address ~0ULiljw肷 srec ̐擪AhX̗p܂B
     */
    long kernel_chk::read_prid( const s_record& srec, unsigned long address )
    {
      const int table_size = 2;
      unsigned long table[table_size];
      long prid = -1;

      if ( address == ~0UL )
      {
        address = srec.lower_bound();
      }

      do
      {
        for ( int i = 0; i < table_size; i++ )
        {
          unsigned long value = 0;
          for ( int j = 0; j < 4; j++ )
          {
            value = value << 8 | srec[address + i*4 + j];
          }
          table[i] = value;
        }

        switch ( table[0] )
        {
        case 0x12345678:
          prid = static_cast<long>( table[1] );
          break;
        case 0x78563412:
          {
            unsigned long value = 0;
            unsigned long t = table[1];
            for ( int j = 0; j < 4; j++ )
            {
              value = value << 8 | ( t & 0xff );
              t >>= 8;
            }
            prid = static_cast<long>( value );
          }
          break;
        default:
          if ( srec.upper_bound() <= address + 1 )
          {
            return -1;
          }
          ++address;
          break;
        }
      } while ( prid < 0 );
      return prid;
    }

    bool kernel_chk::do_save( std::ostream& ostr ) const
    {
      do_file_header( ostr );
      do_body( ostr );
      return true;
    }

    /*!
     *  \brief  load ֐̎
     *  \param  contents  `FbN擾Rec
     */
    void kernel_chk::do_load( const boost::shared_ptr<cfg_contents>& contents )
    {
      typedef cfg_contents::group_container::const_iterator group_iterator;
      const cfg_contents::group_container& container = contents->groups();

      for ( group_iterator giter( contents->groups().begin() ), glast( contents->groups().end() ); giter != glast; ++giter )
      {
        typedef cfg_group::object_container::const_iterator object_iterator;
        const boost::shared_ptr<cfg_group> group( *giter );
        for ( object_iterator oiter( group->objects().begin() ), olast( group->objects().end() ); oiter != olast; ++oiter )
        {
          ++countof_map_[( *oiter )->get_object_type()];
        }
      }
    }

    /*!
     *  \fn     bool kernel_chk::do_load( const s_record& srec, unsigned long address )
     *  \brief  load ֐̎
     *  \param  srec    "kernel_chk.srec"̓e
     *  \param  address ǂݍ݊JnԒn
     *  \return ǂݍ݂ɐꍇtrueԂ
     */

  }
}
