/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*!
 *  \file   toppers/itron/kernel_chk.hpp
 *  \brief  "kernel_chk.c"o͂Ɋւ錾`
 */
#ifndef TOPPERS_ITRON_KERNEL_CHK_HPP_
#define TOPPERS_ITRON_KERNEL_CHK_HPP_

#include "toppers/kernel_id.hpp"
#include <map>
#include <vector>
#include <boost/shared_ptr.hpp>

namespace toppers
{

  class s_record;

  namespace itron
  {

    class cfg_contents;

    /*!
     *  \class  kernel_chk kernel_chk.hpp "toppers/itron/kernel_chk.hpp"
     *  \brief  "kernel_chk.c"o̓NX
     */
    class kernel_chk : public cfg_base
    {
    public:
      enum byteorder_type
      {
        big_endian,
        little_endian,
        unknown = -1
      };

      enum
      {
        magic_number,
        TKERNEL_PRID
      };

      explicit kernel_chk( const std::string& file ) : cfg_base( file ), byteorder_( unknown ) {}
      virtual ~kernel_chk() {}

      /*!
       *  \brief  Rec̃[h
       *  \param  contents  [hRec
       */
      void load( const boost::shared_ptr<cfg_contents>& contents ) { do_load( contents ); }
      void load( const s_record& srec, unsigned long address = ~0UL );

      long countof( const std::string& object_type ) const;
      unsigned long get( int key ) const;
      void set( int key, unsigned long value );
      byteorder_type byteorder() const { return byteorder_; }
      void byteorder( byteorder_type order ) { byteorder_ = order; }

      static long read_prid( const s_record& srec, unsigned long address = ~0UL );
    protected:
      virtual bool do_save( std::ostream& ostr ) const;
      virtual void do_load( const boost::shared_ptr<cfg_contents>& contents );
      virtual bool do_load( const s_record& srec, unsigned long address ) = 0;
    private:
      std::map<std::string, long> countof_map_;
      std::vector<unsigned long> table_;
      byteorder_type byteorder_;
    };

  }
}

#endif  // ! TOPPERS_ITRON_KERNEL_CHK_HPP_
