/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  toppers/itron/kernel_object.cpp
 */
#include "toppers/itron/kernel_object.hpp"
#include "toppers/itron/static_api.hpp"
#include "toppers/parser.hpp"
#include <sstream>
#include <set>
#include <iterator>
#include <algorithm>
#include <limits>

namespace toppers
{
  namespace itron
  {

    /*!
     *  \brief  tB[h̎Q
     *  \param  key   tB[h
     *  \return tB[hɐݒ肳ꂽlԂ
     */
    const std::string& kernel_object::get( const std::string& key ) const
    {
      arg_type::const_iterator iter( args_.find( key ) );
      if ( iter == args_.end() )
      {
        static const std::string dummy;
        return dummy;
      }
      return iter->second;
    }

    /*!
     *  \brief  IuWFNǧ
     *  \param  other   kernel_object ^܂͂̔h^̃IuWFNg
     */
    void kernel_object::swap( kernel_object& other ) throw()
    {
      std::swap( id_, other.id_ );
      name_.swap( other.name_ );
      args_.swap( other.args_ );
    }

    /*!
     *  \brief  p[^Xg̉
     *  \param  api         ÓIAPI
     *  \param  parameters  p[^Xg
     *  \return ͂ɐ true Ԃ
     *
     *  p[^XgɊÂĐÓIAPÏ͂܂B
     *  p[^Xg params ͎̂悤ɁAp[^w肵܂B
     *
     *  \code
     *  // CRE_TSK̏ꍇ
     *  const char cre_tsk_params[] = "$tskid { #tskatr exinf task itskpri stksz stk }";
     *  \endcode
     *
     *  p[^̎ނɉāA̐擪ɋLt邱Ƃł܂B
     *  iITRON 4.0dl 2.1.11 ÓIAPI̎dlƃp[^QƁj
     *
     *  - '$':  tΉp[^
     *  - '%':  tΉp[^
     *  - '#':  vvZbT萔p[^
     *  - Ȃ: ʒ萔p[^
     *
     *  ɁA':'Ŏn܂p[^LqꍇAÓIAPĨp[^ǂݎ̂ł
     *  ȂA':'ɑp[^̒lƂĐݒ肵܂B\n
     *  Ⴆ΁A":RID"ƋLqꍇA":RID"L[ƂtB[h"RID"ݒ肵܂B
     *
     *  pPbg̏ꍇ́A[̒ʁi"{""}"jzɊ܂߂ĂB
     */
    bool kernel_object::analyze_parameters( const static_api& api, const std::string& parameters )
    {
      std::istringstream istr( parameters );
      std::vector<std::string> params;
      typedef std::istream_iterator<std::string> istream_iterator;
      std::copy( istream_iterator( istr ), istream_iterator(), std::back_inserter( params ) );
      static_api::const_iterator iter( api.begin() );
      static_api::const_iterator last( api.end() );

      ++iter; // APIXLbv
      
      typedef std::vector<std::string>::const_iterator params_iterator;
      for ( params_iterator i( params.begin() ), t( params.end() ); i != t; ++i, ++iter )
      {
        std::string param( *i );

        char ch = param[0];
        switch ( ch )
        {
        case '{':
        case '}':
          if ( iter == last || *iter != param )
          {
            static_api::line_type line( api.line() );
            std::string token( ")" );
            if ( iter != last )
            {
              token = *iter;
            }
            error( line.file_.native_file_string(), line.line_, _( "missing `%1%\' before `%2%\'" ) % ch % token );
            return false;
          }
          break;
        case '$': // tΉp[^
        case '%': // tΉp[^
          if ( iter == last || !analyze_id( *iter ) )
          {
            static_api::line_type line( api.line() );
            std::string token( ")" );
            if ( iter != last )
            {
              token = *iter;
            }
            error( line.file_.native_file_string(), line.line_, _( "illegal argument `%1%'" ) % token );
            return false;
          }
          set( param, *iter );
          break;
        case ':':
          set( param, param.substr( 1 ) );
          break;
        case '#': // vvZbT萔p[^
        default:  // ʒ萔p[^
          if ( iter == last )
          {
            static_api::line_type line( api.line() );
            error( line.file_.native_file_string(), line.line_, _( "illegal argument `%1%'" ) % ")" );
            return false;
          }
          set( param, *iter );
          break;
        }
      }
      return true;
    }

    /*!
     *  \brief  tB[h̐ݒ
     *  \param  key   O
     *  \param  value l
     *
     *  key Ŏw肵ÕtB[h value ɐݒ肵܂B
     */
    void kernel_object::set( const std::string& key, const std::string& value )
    {
      args_[key] = value;
    }

    /*!
     *  \brief  ubN̎擾
     *  \return ubNւ̃X}[g|C^Ԃ
     */
    const kernel_object::inib_ptr kernel_object::do_get_inib() const
    {
      std::auto_ptr<kernel_cfg::initial_block> ptr( new kernel_cfg::initial_block );
      ptr->id( id_ );
      if ( !name_.empty() )
      {
        ptr->set( "name", name_ );
        // 荞݂CPUOAIDԍtsȂIuWFNĝ߂̉ԍB
        // ÓIAPIőlgpĂȂꍇɁAIDԍɂȂAR[h
        // Ȃs̑΍
        if ( id_ < 0 )
        {
          ptr->id( std::numeric_limits<long>::max() );
        }
      }
      else
      {
        std::ostringstream osstr;
        osstr << id_;
        ptr->set( "name", osstr.str() );
      }
      for ( arg_type::const_iterator iter( args_.begin() ), last( args_.end() ); iter != last; ++iter )
      {
        ptr->set( iter->first, iter->second );
      }
      return inib_ptr( ptr.release() );
    }

    /*!
     *  \brief  ID̎擾
     *  \return IDւ̃X}[g|C^Ԃ
     */
    const kernel_object::idb_ptr kernel_object::do_get_idb() const
    {
      std::auto_ptr<kernel_id::id_number> ptr( new kernel_id::id_number );
      ptr->set( name_, id_ );
      return idb_ptr( ptr.release() );
    }

    /*!
     *  \brief  IDԍ̉
     *  \param  arg   ͂
     *  \return ͂ɐ true Ԃ
     *
     *  tΉlp[^͂Al̏ꍇ id_ ɐݒ肵A
     *  ʎq̏ꍇ name_ ɐݒ肵܂B
     */
    bool kernel_object::analyze_id( const std::string& arg )
    {
      using namespace boost::spirit;
      id_ = -1L;
      name_.clear();
      if ( parse( arg.begin(), arg.end(), c_int_parser<long>()[ assign_a( id_ ) ] ).full )
      {
        if ( !do_check_id_range( id_ ) )
        {
          return false;
        }
      }
      else
      {
        if ( !parse( arg.begin(), arg.end(), c99_ident_p[ assign_a( name_ ) ] ).full )
        {
          return false;
        }
      }
      return true;
    }

    /*!
     *  \brief  IDԍ̊t⏕֐
     *  \param  first   J[lIuWFNg̐擪ʒu
     *  \param  last    J[lIuWFNg̏I[ʒu+1
     *  \return IDԍ̊tɐ true Ԃ
     */
    bool kernel_object::assign_id_helper( std::vector<kernel_object*>::const_iterator first, std::vector<kernel_object*>::const_iterator last )
    {
      typedef std::vector<kernel_object*> vector_t;
      vector_t::size_type size = last - first;
      vector_t t( size, static_cast<kernel_object*>( 0 ) );
      std::set<std::string> set;

      // 1pXڂŌߑłIDԍ𒲂ׂ
      for ( vector_t::const_iterator iter( first ); iter != last; ++iter )
      {
        kernel_object* pobj = *iter;
        long id = pobj->id();
        if ( id > 0 )
        {
          if ( t[id-1] != 0 )
          {
            error( _( "id `%1%\' of %2% is duplicated" ) % id % pobj->get_object_type() );
            return false;
          }
          t[id-1] = pobj;
        }
        else
        {
          std::string id( pobj->name() );
          if ( !set.insert( id ).second )
          {
            error( _( "id `%1%\' of %2% is duplicated" ) % id % pobj->get_object_type() );
          }
        }
      }
      // 2pXڂŋIDԍԂɊt
      vector_t::size_type pos = 0;
      for ( vector_t::const_iterator iter( first ); iter != last; ++iter )
      {
        kernel_object* pobj = *iter;
        long id = pobj->id();
        if ( id < 0 )
        {
          while ( t[pos] != 0 )
          {
            ++pos;
          }
          t[pos] = pobj;
          pobj->id_ = pos+1;
        }
      }
      return true;
    }

    /*!
     *  \fn   bool kernel_object::do_analyze( const static_api& api )
     *  \brief  analyze ֐̎
     *  \param  api   ͂ÓIAPI
     *  \return ͂ɐ true Ԃ
     */

    /*!
     *  \fn   const char* kernel_object::do_get_object_type() const
     *  \brief  J[xIuWFNg̎ނ̎Q
     *  \return ނ\Ԃ
     */

    /*!
     *  \fn   bool kernel_object::do_has_id() const
     *  \return IDԍgpꍇɂ true Ԃ
     *  \note "do has"Ƃ͉̂pIɂ͊ԈႢłA̕֋X̂߂
     *        Ă̂悤ɖĂ܂B
     */

  }
}
