/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*!
 *  \file   toppers/itron/kernel_object.hpp
 *  \brief  J[lIuWFNgɊւ錾`
 *
 *  ̃t@CŒ`NX
 *  \code
 *  class kernel_object;
 *  \endcode
 */
#ifndef TOPPERS_KERNEL_OBJECT_HPP_
#define TOPPERS_KERNEL_OBJECT_HPP_

#include "toppers/itron/cfg_kernel_id.hpp"
#include "toppers/itron/cfg_kernel_cfg.hpp"

namespace toppers
{
  namespace itron
  {

    class static_api;

    /*!
     *  \class  kernel_object kernel_object.hpp "toppers/itron/kernel_object.hpp"
     *  \brief  J[lIuWFNgǗNX
     */
    class kernel_object
    {
    public:
      typedef boost::shared_ptr<kernel_cfg::initial_block> inib_ptr;
      typedef boost::shared_ptr<kernel_id::id_number> idb_ptr;
      typedef std::map<std::string, std::string> arg_type;
      typedef const boost::shared_ptr<kernel_object> ( *builder_type )( const static_api&, std::vector<boost::shared_ptr<kernel_object> >& );

      /*!
       *  \brief  ftHgRXgN^
       */
      kernel_object() : id_( -1L ) {}
      /*!
       *  \brief  fXgN^
       */
      virtual ~kernel_object() {}

      /*!
       *  \brief  ubN̎擾
       *  \return ubNւ̃X}[g|C^Ԃ
       */
      const inib_ptr get_inib() const { return do_get_inib(); }
      /*!
       *  \brief  IDԍ̎擾
       *  \return IDԍւ̃X}[g|C^Ԃ
       */
      const idb_ptr get_idb() const { return do_get_idb(); }
      const char* get_object_type() const { return do_get_object_type(); }
      bool has_id() const { return do_has_id(); }

      /*!
       *  \brief  IDԍ̎擾
       *  \return IDԍԂ
       */
      long id() const { return id_; }
      /*!
       *  \brief  IuWFNg̎擾
       *  \return IuWFNgԂ
       */
      const std::string& name() const { return name_; }
      const std::string& get( const std::string& key ) const;
      void set( const std::string& key, const std::string& value );

      /*!
       *  \brief  ÓIAPỈ
       *  \param  api   ÓIAPĨp[^
       *  \return ͂ɐ true Ԃ
       */
      bool analyze( const static_api& api ) { return do_analyze( api ); }
      bool analyze_id( const std::string& arg );
      bool analyze_parameters( const static_api& api, const std::string& parameters );

      void swap( kernel_object& other ) throw();

      /*!
       *  \brief  IDԍɎt
       *  \param  first   kernel_object Rei̐擪ʒu
       *  \param  last    kernel_object ReȉI[ʒu+1
       *  \return IDԍ̊tɐ true Ԃ
       */
      template <class Iterator>
      static bool assign_id( Iterator first, Iterator last )
      {
        typedef std::vector<kernel_object*> vector_t;
        typedef std::map<std::string, vector_t> map_t;
        map_t map;
        for ( Iterator iter( first ); iter != last; ++iter )
        {
          std::string key( ( *iter )->get_object_type() );
          kernel_object* ptr = &**iter;
          map[key].push_back( ptr );
        }
        for ( map_t::const_iterator iter2( map.begin() ), last2( map.end() ); iter2 != last2; ++iter2 )
        {
          if ( !iter2->second.empty() && iter2->second.front()->has_id() )  // IDԍtKvȏꍇɌ
          {
            if ( !assign_id_helper( iter2->second.begin(), iter2->second.end() ) )
            {
              return false;
            }
          }
        }
        return true;
      }

    protected:
      virtual bool do_analyze( const static_api& api ) = 0;
      virtual const inib_ptr do_get_inib() const;
      virtual const idb_ptr do_get_idb() const;
      virtual const char* do_get_object_type() const = 0;
      virtual bool do_has_id() const = 0;
      virtual bool do_check_id_range( long id ) const = 0;

      static bool assign_id_helper( std::vector<kernel_object*>::const_iterator first, std::vector<kernel_object*>::const_iterator last );
    private:
      long id_;
      std::string name_;
      arg_type args_;
    };

  }
}

#endif  // ! TOPPERS_KERNEL_OBJECT_HPP_
