/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*!
 *  \file   toppers/itron/specified_factory.hpp
 *  \brief  J[lt@Ng[Ɋւ錾`
 *
 *  ̃t@CŒ`NX
 *  \code
 *  class specified_factory<Policy>;
 *  \endcode
 */
#ifndef TOPPERS_ITRON_SPECIFIED_FACTORY_HPP_
#define TOPPERS_ITRON_SPECIFIED_FACTORY_HPP_

#include "toppers/itron/cfg_factory.hpp"
#include <boost/mpl/for_each.hpp>

namespace toppers
{
  namespace itron
  {

    template <class Policy> class specified_object;

    /*!
     *  \brief  boost::mpl::for_each ɗ^t@N^`}N
     *  \param  name    NX
     *  \param  result  ʂ̌^iʏA֐ւ̃|C^j
     *  \param  func    specified_object<Policy> NX̃o֐
     *
     *  specified_factory<Policy> NXɂ Policy::sequence ɑ΂ boost::mpl::for_each
     *  ̌ďoɕKvȃt@N^NX`܂B
     */
#define TOPPERS_DEFINE_CFG_FACTORY_PRED( name, result, func ) \
          struct name { \
            typedef result result_type;  \
            name() : result_( new std::vector<result_type> ) {} \
            template <typename T> void operator()( const T& ) const { \
              result_->push_back( &specified_object<T>::func ); \
            } \
            boost::shared_ptr<std::vector<result_type> > result_; \
          }


    /*!
     *  \class  specified_factory specified_factory.hpp "toppers/itron/specified_factory.hpp"
     *  \brief  J[lt@Ng[𐶐邽߂̃ev[gNX
     *
     *  ev[g Policy ɂ͎̃o^w肵ĂB
     *
     *  \code
     *  // toppers::itron::task_policy ̃J[lIuWFNg̃|V[NX̌^V[PX
     *  // ʏ include_policy 擪vfɎw肷邱ƁB
     *  typedef boost::mpl::list<...> sequence;
     *
     *  // |V[T|[gJ[lp kernel_chk NX
     *  typedef << toppers::itron::kernel_chk ̔hNX>> kernel_chk;
     *
     *  // "-kernel"IvVŎw肵񂪁A|V[T|[gJ[lɍv
     *  // ǂ̔֐BvꍇtrueԂB
     *  static bool match_version( const std::string& ver );
     *
     *  // ̃J[lIuWFNgɑȂJ[lŗL̏R[h̐֐
     *  // JSPJ[l̏ꍇA _kernel_tmevt_heap z̐ɑB
     *  static void extra_init( const kernel_cfg& cfg, std::ostream& ostr );
     *
     *  // TKERNEL_PRID}N̒l
     *  static long prid_;
     *  \endcode
     */
    template <class Policy>
    class specified_factory : public cfg_factory
    {
    public:
      /*!
       *  \brief  J[lIuWFNg|V[^Xg
       *
       *  Boost.MPL ɂ^V[PXi boost::mpl::list<> j\܂B
       */
      typedef typename Policy::sequence sequence;

      /*!
       *  \brief  RXgN^
       *  \param  options   R}hCIvV
       *  \param  version   J[lwo[W
       *  \param  prid      TKERNEL_PRID }N̒l
       */
      specified_factory( const cmdline::option_array& options, const std::string& version, long prid )
        : cfg_factory( options, version, prid )
      {
      }

      /*!
       *  \brief  t@Ng̍\z
       *  \param  options   R}hCIvV
       *  \param  version   J[lwo[W
       *  \return \zɐ΃t@Ngւ̃X}[g|C^Ԃ
       *
       *  ̊֐ version Ŏw肳ꂽo[W specified_factory<Policy> ̂̂
       *  v΃t@Ng\z܂B
       */
      static const boost::shared_ptr<cfg_factory> build( const cmdline::option_array& options, const std::string& version )
      {
        boost::shared_ptr<cfg_factory> ptr;
        if ( Policy::match_version( version ) )
        {
          ptr.reset( new specified_factory<Policy>( options, version, Policy::prid_ ) );
        }
        return ptr;
      }
    protected:
      /*!
       *  \brief  eJ[lIuWFNg̊֐Q̎擾
       *  \param  pred  ֐擾̂߂̏q
       *  \return ֐Qi[ std::vector ւ̃X}[g|C^Ԃ
       *
       *  Pred ɂ͎̃oNX^w肵ĂB
       *  \code
       *  typedef <<֐ւ̃|C^^>> result_type;
       *  template <typename T> void operator()( const T& ) const;
       *  boost::shared_ptr<std::vector<result_type> > result_;
       *  \endcode
       */
      template <class Pred>
      static const boost::shared_ptr<std::vector<typename Pred::result_type> > get_funcions( Pred pred )
      {
        boost::mpl::for_each< sequence >( pred );
        return pred.result_;
      }

      virtual const boost::shared_ptr<std::vector<kernel_cfg_generator_type> > do_create_kernel_cfg_generators() const
      {
        pred_kernel_cfg pred;
        return get_funcions( pred );
      }
      virtual const boost::shared_ptr<std::vector<kernel_id_generator_type> > do_create_kernel_id_generators() const
      {
        pred_kernel_id pred;
        return get_funcions( pred );
      }
      virtual const boost::shared_ptr<std::vector<kernel_object::builder_type> > do_get_kernel_object_builders() const
      {
        pred_kernel_object pred;
        return get_funcions( pred );
      }
      virtual const boost::shared_ptr<std::vector<std::string> > do_get_kernel_object_names() const
      {
        pred_kernel_object_name pred;
        boost::mpl::for_each< sequence >( pred );
        return pred.result_;
      }
      virtual extra_initializer_type do_get_extra_initializer() const
      {
        return &Policy::extra_init;
      }
      virtual const boost::shared_ptr<kernel_chk> do_create_kernel_chk() const
      {
        return boost::shared_ptr<kernel_chk>( new typename Policy::kernel_chk( "kernel_chk.c" ) );
      }
    private:
      TOPPERS_DEFINE_CFG_FACTORY_PRED( pred_kernel_cfg, kernel_cfg_generator_type, generate_kernel_cfg );
      TOPPERS_DEFINE_CFG_FACTORY_PRED( pred_kernel_id, kernel_id_generator_type, generate_kernel_id );
      TOPPERS_DEFINE_CFG_FACTORY_PRED( pred_kernel_object, kernel_object::builder_type, build );

      struct pred_kernel_object_name
      {
        typedef std::string result_type;
        pred_kernel_object_name() : result_( new std::vector<result_type> ) {}
        template <typename T> void operator()( const T& ) const
        {
          result_->push_back( T::name_ );
        }
        boost::shared_ptr<std::vector<result_type> > result_;
      };
    };

  }
}

#endif  // ! TOPPERS_ITRON_SPECIFIED_FACTORY_HPP_
