/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*!
 *  \file   toppers/itron/static_api_parser.hpp
 *  \brief  ÓIAPI̕@Ɋւ錾`
 *
 *  ̃t@CŒ`NX
 *  \code
 *  struct static_api_parser;
 *  struct static_api_parser::error_handler;
 *  struct static_api::definition<Scanner>;
 *  \endcode
 */
#ifndef TOPPERS_STATIC_API_PARSER_HPP_
#define TOPPERS_STATIC_API_PARSER_HPP_

#include "toppers/itron/static_api.hpp"
#include "toppers/c_expr.hpp"
#include <boost/spirit/error_handling.hpp>

namespace toppers
{
  namespace itron
  {

    /*!
     *  \struct static_api_parser static_api_parser.hpp "toppers/itron/static_api_parser.hpp"
     *  \brief  ÓIAPI̍\̓NX
     */
    struct static_api_parser : boost::spirit::grammar<static_api_parser>
    {
      enum rule_id_t
      {
        id_top = 1, id_api_name, id_parameter_list, id_parameter, id_packet, id_cexpr
      };
      enum expected_t
      {
        open_paren_expected, close_paren_expected, open_brace_expected, close_brace_expected,
        comma_expected, semicolon_expected
      };

      /*!
       *  \struct error_handler static_api_parser.hpp "toppers/itron/static_api_parser.hpp"
       *  \brief  ÓIAPI̍\͂ɂG[t@N^
       */
      struct error_handler
      {
        template <class Scanner, class Error>
        boost::spirit::error_status<> operator()( const Scanner& scan, const Error& error ) const
        {
          typename Error::iterator_t iter( error.where );
          while ( *iter != '\0' && *iter != '\n' )
          {
            ++iter;
          }
          std::string str( error.where, iter );
          str = '\"' + str + '\"';

          text_line ln( get_text_line( error.where ) );
          switch ( error.descriptor )
          {
          case open_paren_expected:
            toppers::error( ln, _( "missing `%1%\' before %2%" ) % '(' % str );
            break;
          case close_paren_expected:
            toppers::error( ln, _( "missing `%1%\' before %2%" ) % ')' % str );
            break;
          case close_brace_expected:
            toppers::error( ln, _( "missing `%1%\' before %2%" ) % '}' % str );
            break;
          case semicolon_expected:
            toppers::error( ln, _( "missing `%1%\' before %2%" ) % ';' % str );
            break;
          }
          return  boost::spirit::error_status<>( boost::spirit::error_status<>::fail );
        }
      };

      /*!
       *  \struct definition static_api_parser.hpp "toppers/itron/static_api_parser.hpp" 
       *  \brief  ÓIAPI̍\͂ɂ镶@`
       */
      template <class Scanner>
      struct definition
      {
        typedef boost::spirit::rule<Scanner, boost::spirit::dynamic_parser_tag> rule_t;
        typedef boost::spirit::guard<expected_t> guard_t;
        typedef boost::spirit::assertion<expected_t> assertion_t;

        const c_strlit_parser_t c_strlit_p;
        const c_ident_parser_t c_ident_p;

        rule_t  top, api_name, parameter_list, parameter, packet, cexpr;
        guard_t guard_api, guard_packet;
        assertion_t expect_open_paren,
                    expect_close_paren,
                    expect_close_brace,
                    expect_comma, expect_semicolon;

        /*!
         *  \brief  RXgN^
         *  \param  self  \̓NXi@NXjւ̎Q
         */
        definition( const static_api_parser& self )
          : c_strlit_p( c_strlit_parser( self.cexpr_p_.codeset_ ) ),
            c_ident_p( c_ident_parser( self.cexpr_p_.ucn_, self.cexpr_p_.codeset_ ) ),
            expect_open_paren( open_paren_expected ),
            expect_close_paren( close_paren_expected ),
            expect_close_brace( close_brace_expected ),
            expect_comma( comma_expected ),
            expect_semicolon( semicolon_expected )
        {
          using namespace boost::spirit;
          set_id();
          top =
              guard_api
              (
                api_name >> 
//                  expect_open_paren( str_p( "(" ) ) >>  // local_classwqŃG[ɂȂ̂Ŕ
                  str_p( "(" ) >>
                  parameter_list >>
                  expect_close_paren( str_p( ")" ) ) >>
                  expect_semicolon( ch_p( ';' ) )
              )
              [
                error_handler()
              ];
          api_name =
              c_ident_p[ push_back_a( self.tokens_ ) ];
          parameter_list =
              parameter % ',';
          parameter =
              packet | cexpr;
          packet =
              guard_packet
              (
                str_p( "{" )[ push_back_a( self.tokens_ ) ] >>
                parameter_list >>
                expect_close_brace( str_p( "}" )[ push_back_a( self.tokens_ ) ] ) 
              )
              [
                error_handler()
              ];
          cexpr =
              self.cexpr_p_[ push_back_a( self.tokens_ ) ];
        }
        void set_id()
        {
          top.set_id( id_top );
          api_name.set_id( id_api_name );
          parameter_list.set_id( id_parameter_list );
          parameter.set_id( id_parameter );
          packet.set_id( id_packet );
          cexpr.set_id( id_cexpr );
        }
        const rule_t& start() const { return top; }
      };
      /*!
       *  \brief  RXgN^
       *  \param  tokens  ÓIAPI̍\g[N̊i[
       *  \param  cexpr_p C̒萔\͊֐IuWFNg
       */
      explicit static_api_parser( std::vector<std::string>& tokens, const c_const_expr_parser& cexpr_p )
        : tokens_( tokens ), cexpr_p_( cexpr_p )
      {
      }

      std::vector<std::string>& tokens_;
      const c_const_expr_parser& cexpr_p_;
    };

  }
}

#endif  // ! TOPPERS_STATIC_API_PARSER_HPP_
