/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2004 by Witz Corporation, JAPAN  
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*!
 *  \file   toppers/kernel_cfg.hpp
 *  \brief  "kernel_cfg.c"t@Co͂Ɋւ錾
 *
 *  ̃t@CŒ`NX
 *  \code
 *  class kernel_cfg;
 *  class kernel_cfg::initial_block;
 *  \endcode
 */
#ifndef TOPPERS_KERNEL_CFG_HPP_
#define TOPPERS_KERNEL_CFG_HPP_

#include "toppers/kernel_id.hpp"
#include <iosfwd>
#include <map>
#include <vector>
#include <string>
#include <boost/shared_ptr.hpp>

namespace toppers
{

  /*!
   *  \class  kernel_cfg kernel_cfg.hpp "toppers/kernel_cfg.hpp"
   *  \brief  "kernel_cfg.c"t@Co͂̂߂̊NX
   *  \sa kernel_id
   */
  class kernel_cfg : public cfg_base
  {
  public:
    /*!
     *  \class  initial_block kernel_cfg.hpp "toppers/kernel_cfg.hpp"
     *  \brief  J[lIuWFNg̏ubNNX
     *  \sa kernel_cfg::initial_block, kernel_id
     */
    class initial_block
    {
    public:
      initial_block();
      /*!
       *  \brief  fXgN^
       */
      virtual ~initial_block() {}

      const std::string& get( const std::string& key ) const;
      void set( const std::string& key, const std::string& field );
      long id() const;
      void id( long value );

      static bool less_than( const boost::shared_ptr<initial_block> lhs, const boost::shared_ptr<initial_block> rhs );
      static const std::string& get_field( const boost::shared_ptr<initial_block>& inib_ptr, const std::string& key );
    private:
      std::map<std::string, std::string> field_map_;
      long id_;
    };

    typedef std::vector<boost::shared_ptr<initial_block> > inib_array;

    explicit kernel_cfg( const std::string& file = std::string( "kernel_cfg.c" ),
                         const std::string& id_file = std::string( "kernel_id.h" ) );
    const std::string& id_file() const;
    const inib_array* get_inib_array( const std::string& key ) const;
  protected:
    typedef std::map<std::string, inib_array> inib_array_map;

    void insert_inib( const std::string& key, const boost::shared_ptr<initial_block>& inib );
    void clear_inib_array( const std::string& key );

    virtual bool do_save( std::ostream& ostr ) const;
  private:
    std::string id_file_;
    inib_array_map inib_map_;
  };

}

#endif  // ! TOPPERS_KERNEL_CFG_HPP_
