/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2004 by Witz Corporation, JAPAN  
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*!
 *  \file   toppers/kernel_id.hpp
 *  \brief  "kernel_id.h"t@C̏o͂Ɋւ錾
 *
 *  ̃t@CŒ`NX
 *  \code
 *  class cfg_base;
 *    class kernel_id;
 *  class kernel_id::id_number;
 *  \endcode
 */
#ifndef TOPPERS_KERNEL_ID_HPP_
#define TOPPERS_KERNEL_ID_HPP_

#include "toppers/config.hpp"
#include <iosfwd>
#include <string>
#include <map>
#include <vector>
#include <boost/shared_ptr.hpp>

namespace toppers
{

  /*!
   *  \class  cfg_base kernel_id.hpp "toppers/kernel_id.hpp"
   *  \brief  ݒt@Co͂̂߂̊NX
   */
  class cfg_base
  {
  public:
    explicit cfg_base( const std::string& file );
    /*!
     *  \brief  fXgN^
     */
    virtual ~cfg_base() {}
    void save() const;
    const std::string file() const;
  protected:
    virtual bool do_save( std::ostream& ostr ) const = 0;
    virtual void do_body( std::ostream& ostr ) const = 0;
    virtual void do_file_header( std::ostream& ostr ) const;
    virtual void do_depend() const;
  private:
    std::string file_;
  };

  /*!
   *  \class  kernel_id kernel_id.hpp "toppers/kernel_id.hpp"
   *  \brief  "kernel_id.h"t@Co͂̂߂̊NX
   *  \sa kernel_id::id_number, kernel_cfg
   */
  class kernel_id : public cfg_base
  {
  public:
    /*!
     *  \class  id_number kernel_id.hpp "toppers/kernel_id.hpp"
     *  \brief  IuWFNgIDԍubN̊NX
     */
    class id_number
    {
    public:
      id_number();
      /*!
       *  \brief  fXgN^
       */
      virtual ~id_number() {}

      const std::string& name() const;
      long value() const;
      void set( const std::string& name, long id );

      static bool less_than( const boost::shared_ptr<id_number> lhs, const boost::shared_ptr<id_number> rhs );
    private:
      std::string name_;
      long id_;
    };

    typedef std::vector<boost::shared_ptr<id_number> > id_array;

    explicit kernel_id( const std::string& file = std::string( "kernel_id.h" ) );

    const id_array* get_id_array( const std::string& key ) const;
  protected:
    void insert_id( const std::string& key, const boost::shared_ptr<id_number>& id );
    void clear_id_array( const std::string& key );

    virtual bool do_save( std::ostream& ostr ) const;

    static const std::string include_guard_macro( const std::string& file );
  private:
    std::map<std::string, id_array> id_map_;
  };

}

#endif  // ! TOPPERS_KERNEL_ID_HPP_
