/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*!
 *  \file   toppers/misc.hpp
 *  \brief  GȃCû߂̐錾`
 */
#ifndef TOPPERS_MISC_HPP_
#define TOPPERS_MISC_HPP_

#include "toppers/config.hpp"
#include <stdexcept>
#include <locale>
#include <iosfwd>
#include <string>
#include <algorithm>
#include <cctype>
#include <cwchar>
#include <cstdlib>
#include <boost/scoped_array.hpp>

namespace toppers
{

  /*!
   *  \class  conversion_error misc.hpp "toppers/misc.hpp"
   *  \brief  ϊG[ONX
   */
  class conversion_error : public std::runtime_error
  {
  public:
    /*!
     *  \brief  RXgN^
     *  \param  what  O
     */
    explicit conversion_error( const std::string& what ) : std::runtime_error( what ) {}
  };

  /*!
   *  \brief  VOoCg CharT ^i wchar_t ^jւ̕ϊ
   *  \param  ch  VOoCg
   *  \return CharT ^Ԃ
   */
  template <typename CharT>
  inline CharT widen( char ch )
  {
    return std::use_facet<std::ctype<CharT> >( std::locale() ).widen( ch );
  }

  template <>
  inline char widen<char>( char ch )
  {
    return ch;
  }

  template <>
  inline unsigned char widen<unsigned char>( char ch )
  {
    return static_cast<unsigned char>( ch );
  }

  template <>
  inline wchar_t widen<wchar_t>( char ch )
  {
#if defined( __MINGW32__ )
    wchar_t wc = wchar_t( -1 );
    std::mbtowc( &wc, &ch, 1 ) < 0 )
    return wc;
#elif defined( __CYGWIN__ )
    return static_cast<wchar_t>( static_cast<unsigned char>( ch ) );
#else
    return static_cast<wchar_t>( std::btowc( static_cast<unsigned char>( ch ) ) );
#endif
  }

  /*!
   *  \brief  VOoCg񂩂 CharT ^ւ̕ϊ
   *  \param  str   VOoCg
   *  \return CharT ^Ԃ
   */
  template <typename CharT>
  const std::basic_string<CharT> widen( const std::string& str );

  template <>
  inline const std::basic_string<char> widen( const std::string& str )
  {
    return str;
  }

  template <>
  inline const std::basic_string<wchar_t> widen( const std::string& str )
  {
    boost::scoped_array<wchar_t> t( new wchar_t[str.size()+1] );
    if ( std::mbstowcs( t.get(), str.c_str(), str.size() ) == size_t( -1 ) )
    {
      static conversion_error x( "in function widen" );
      throw x;
    }
    return t.get();
  }

#undef  tolower

  /*!
   *  \brief  ւ̕ϊ
   *  \param  ch    ϊΏۂ̕
   *  \return ch 啶łΑΉ鏬AȊO ch Ԃ
   */
  inline char tolower( char ch )
  {
    return static_cast<char>( std::tolower( static_cast<unsigned char>( ch ) ) );
  }

  /*!
   *  \brief  ɕϊ
   *  \param  str   ϊΏۂ̕
   *  \return str Ɋ܂܂啶ɕϊԂ
   */
  inline const std::string tolower( std::string str )
  {
    char ( *f )( char ch ) = &tolower;
    std::transform( str.begin(), str.end(), str.begin(), f );
    return str;
  }

  /*!
   *  \brief  Chɕϊ
   *  \param  str   ϊΏۂ̕
   *  \return str Ɋ܂܂啶ɕϊChԂ
   */
  template <class Traits, class Allocator>
  const std::basic_string<wchar_t, Traits, Allocator> tolower( std::basic_string<wchar_t, Traits, Allocator> str )
  {
    std::transform( str.begin(), str.end(), str.begin(), &std::towlower );
    return str;
  }

#undef  toupper

  /*!
   *  \brief  啶ւ̕ϊ
   *  \param  ch    ϊΏۂ̕
   *  \return ch łΑΉ啶AȊO ch Ԃ
   */
  inline char toupper( char ch )
  {
    return static_cast<char>( std::toupper( static_cast<unsigned char>( ch ) ) );
  }

  /*!
   *  \brief  啶ɕϊ
   *  \param  str   ϊΏۂ̕
   *  \return str Ɋ܂܂鏬啶ɕϊԂ
   */
  inline const std::string toupper( std::string str )
  {
    char ( *f )( char ) = &toupper;
    std::transform( str.begin(), str.end(), str.begin(), f );
    return str;
  }

  /*!
   *  \brief  Ch啶ɕϊ
   *  \param  str   ϊΏۂ̕
   *  \return str Ɋ܂܂鏬啶ɕϊChԂ
   */
  template <class Traits, class Allocator>
  const std::basic_string<wchar_t, Traits, Allocator> toupper( std::basic_string<wchar_t, Traits, Allocator> str )
  {
    std::transform( str.begin(), str.end(), str.begin(), &std::towupper );
    return str;
  }

  /*!
   *  \brief  w蕶ŋ؂ꂽXgo
   *  \param  first   o͂擪vfʒu
   *  \param  last    o͂I[vfʒu+1
   *  \param  ostr    o̓Xg[
   *  \param  pred    evf󂯎o͒lԂq
   *  \param  delim   ؕ
   *
   *  ̊֐͋ [first, last) ̊evf pred ɓnēl delim
   *  ŋ؂ ostr ɏo͂܂B\n
   *  I[̗vf̌ɂ delim ͏o͂ꂸAvfƗvf̊Ԃɂ̂ delim 
   *  o͂܂B
   */
  template <class InputIterator, typename CharT, class Traits, class Pred>
  void output_list( InputIterator first, InputIterator last, std::basic_ostream<CharT, Traits>& ostr, Pred pred, const CharT* delim = 0 )
  {
    if ( delim == 0 )
    {
      static const CharT null_delim[] = { 0 };
      delim = null_delim;
    }

    for ( bool f = false; first != last; ++first )
    {
      if ( f )
      {
        ostr << delim;
      }
      ostr << pred( *first );
      f = true;
    }
  }

}

#endif  // ! TOPPERS_MISC_HPP_
