/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2004 by Witz Corporation, JAPAN  
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*
 *  toppers/reporter.cpp
 */
#include "toppers/reporter.hpp"
#include <iostream>

namespace toppers
{

  namespace
  {
    const char* program_name = "cfg";
  }

  /*!
   *  \brief  ̏o͐
   *  \sa set_default_stream
   *
   *  ̃oɐݒ肳ꂽo̓Xg[ɃbZ[W񂪏o͂܂B
   */
  std::ostream* reporter::default_stream_ = &std::cerr;

  /*!
   *  \brief  |֐
   *  \sa set_translator
   *
   *  ̃oɐݒ肳ꂽ|֐ɃbZ[W񂪓n܂B
   */
  reporter::translator_type reporter::translator_ = 0;

  /*!
   *  \brief  abort Ɏ܂ł̃G[
   *  
   *  error_count_ ̏l̉񐔂G[_ reporter::abort ֐
   *  Ăяo܂B
   */
  int reporter::error_count_ = 1;

  /*!
   *  \brief  RXgN^
   *  \param  stream  bZ[W̏o͐
   */
  reporter::reporter( std::ostream* stream ) : stream_( stream )
  {
    if ( stream_ == 0 )
    { 
      stream_ = default_stream_;
    }
  }

  /*!
   *  \brief  fXgN^
   */
  reporter::~reporter()
  { 
    stream_ = 0;
  }

  /*!
   *  \brief  x
   *  \param  fmt   o̓bZ[W
   */
  void reporter::warning( const boost::format& fmt ) const
  { 
    *stream_ << program_name << ": " << _( "warning: " ) << fmt << std::endl; 
  }

  /*!
   *  \brief  xisԍj
   *  \param  file  xt@C
   *  \param  line  xsԍ
   *  \param  fmt   o̓bZ[W
   */
  void reporter::warning( const std::string& file, int line, const boost::format& fmt ) const
  { 
    *stream_ << program_name << ": " << file << ':' << line << ": " << _( "warning: " ) << fmt << std::endl; 
  }

  /*!
   *  \brief  G[
   *  \param  fmt   o̓bZ[W
   *  \throw  toppers::reporter::terminator
   */
  void reporter::error( const boost::format& fmt ) const
  {
    *stream_ << program_name << ": " << _( "error: " ) << fmt << std::endl;
    if ( --error_count_ <= 0 )
    {
      abort();
    }
  }

  /*!
   *  \brief  G[isԍj
   *  \param  file  G[t@C
   *  \param  line  G[sԍ
   *  \param  fmt   o̓bZ[W
   *  \throw  toppers::reporter::terminator
   */
  void reporter::error( const std::string& file, int line, const boost::format& fmt ) const
  {
    *stream_ << program_name << ": " << file << ':' << line << ": " << _( "error: " ) << fmt << std::endl;
    if ( --error_count_ <= 0 )
    {
      abort();
    }
  }

  /*!
   *  \brief  vIG[
   *  \param  fmt   o̓bZ[W
   *  \throw  toppers::reporter::terminator
   */
  void reporter::fatal( const boost::format& fmt ) const
  {
    *stream_ << program_name << ": " << _( "fatal error: " ) << fmt << std::endl;
    abort();
  }

  /*!
   *  \brief  vIG[isԍj
   *  \param  file  vIG[t@C
   *  \param  line  vIG[sԍ
   *  \param  fmt   o̓bZ[W
   *  \throw  toppers::reporter::terminator
   */
  void reporter::fatal( const std::string& file, int line, const boost::format& fmt ) const
  {
    *stream_ << program_name << ": " << file << ':' << line << ": " << _( "fatal error: " ) << fmt << std::endl;
    abort();
  }

  /*!
   *  \brief  vO̒~
   *  \param  fmt   o̓bZ[W
   *  \throw  toppers::reporter::terminator
   */
  void reporter::stop( const boost::format& fmt ) const
  {
    *stream_ << program_name << ": " << fmt << std::endl;
    abort();
  }

  /*!
   *  \brief  vO̐ݒ
   *  \param  name  vO
   */
  void reporter::set_program_name( const char* name )
  {
    program_name = name;
  }

  /*!
   *  \brief  ̏o͐̐ݒ
   *  \param  ostr  o̓Xg[
   */
  void reporter::set_default_stream( std::ostream* ostr )
  { 
    default_stream_ = ostr; 
  }

  /*!
   *  \brief  bZ[W̖|
   *  \param  str   |Ώۂ̕  
   *  \return |̕Ԃ
   *  \sa _, set_translator
   *
   *  _()}NɂāA|󂷂邽߂ɌĂяo܂B
   */
  const char* reporter::translate( const char* str )
  {
    if ( translator_ != 0 )
    { 
      return ( *translator_ ) ( str ); 
    }
    return str;
  }

  /*!
   *  \brief  |֐̓o^
   *  \param  func  o^|֐ւ̃|C^
   *  \return ȑOɓo^Ă|֐ւ̃|C^Ԃ
   *  \sa translate
   */
  reporter::translator_type reporter::set_translator( translator_type func )
  {
    translator_type prev = translator_;
    translator_ = func;
    return prev;
  }

  /*!
   *  \brief  ُI
   *
   *  IuWFNg̃fXgN^Ăяo悤ɗO𑗏o邱ƂŁA
   *  vOI܂B
   *  main֐toppers::reporter::terminatorcatchŁAK؂ɃvO
   *  IĂB
   *
   *  \throw  toppers::reporter::terminator
   */
  void reporter::abort() const
  { 
    throw terminator(); 
  }

  void warning( const text_line& line, const boost::format& fmt )
  {
    reporter r;
    if ( line.line_ > 0 )
    {
      r.warning( line.file_.native_file_string(), line.line_, fmt );
    }
    else
    {
      r.warning( fmt );
    }
  }

  void error( const text_line& line, const boost::format& fmt )
  {
    reporter r;
    if ( line.line_ > 0 )
    {
      r.error( line.file_.native_file_string(), line.line_, fmt );
    }
    else
    {
      r.error( fmt );
    }
  }

  void fatal( const text_line& line, const boost::format& fmt )
  {
    reporter r;
    if ( line.line_ > 0 )
    {
      r.fatal( line.file_.native_file_string(), line.line_, fmt );
    }
    else
    {
      r.fatal( fmt );
    }
  }

}
