/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2004 by Witz Corporation, JAPAN  
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*!
 *  \file   toppers/reporter.hpp
 *  \brief  oߕ񍐂Ɋւ錾`
 *
 *  ̃t@CŒ`NX
 *  \code
 *  class reporter;
 *  class reporter::terminator;
 *  \endcode
 */
#ifndef	TOPPERS_REPORTER_HPP_
#define	TOPPERS_REPORTER_HPP_

#include "toppers/text_line.hpp"
#include <iosfwd>
#include <string>
#include <boost/format.hpp>

namespace toppers
{

  /*!
   *  \class  reporter reporter.hpp "toppers/reporter.hpp"
   *  \brief  o߂̕񍐃NX
   */
  class reporter
  {
  public:
    /*!
     *  \class  terminator reporter.hpp "toppers/reporter.hpp"
     *  \brief  G[̗ONX
     *  \sa reporter
     */
    class terminator {};

    enum language_type
    {
      English, Japanese,
      number_of_languages
    };

    typedef const char* ( *translator_type )( const char* );

    explicit reporter( std::ostream* stream = 0 );
    virtual ~reporter();
    void warning( const boost::format& fmt ) const;
    void warning( const std::string& file, int line, const boost::format& fmt ) const;
    void error( const boost::format& fmt ) const;
    void error( const std::string& file, int line, const boost::format& fmt ) const;
    void fatal( const boost::format& fmt ) const;
    void fatal( const std::string& file, int line, const boost::format& fmt ) const;
    void stop( const boost::format& fmt ) const;
     
    static void set_program_name( const char* name );
    static void set_default_stream( std::ostream* ostr );
    static const char* translate( const char* str );
    static translator_type set_translator( translator_type func );
  protected:
    void abort() const;
  private:
    std::ostream* stream_;
    static std::ostream* default_stream_;
    static translator_type translator_;
    static int error_count_;
    static std::string perhaps_error_;
  };

  /*!
   *  \brief  x
   *  \param  fmt   o̓bZ[W
   */
  inline void warning( const boost::format& fmt )
  {
    reporter r;
    r.warning( fmt );
  }
  /*!
   *  \brief  xisԍj
   *  \param  file  xt@C
   *  \param  line  xsԍ
   *  \param  fmt   o̓bZ[W
   */
  inline void warning( const std::string& file, int line, const boost::format& fmt )
  {
    reporter r;
    r.warning( file, line, fmt );
  }
  void warning( const text_line& line, const boost::format& fmt );

  /*!
   *  \brief  G[
   *  \param  fmt   o̓bZ[W
   *  \throw  toppers::reporter::terminator
   */
  inline void error( const boost::format& fmt )
  {
    reporter r;
    r.error( fmt );
  }
  /*!
   *  \brief  G[isԍj
   *  \param  file  G[t@C
   *  \param  line  G[sԍ
   *  \param  fmt   o̓bZ[W
   *  \throw  toppers::reporter::terminator
   */
  inline void error( const std::string& file, int line, const boost::format& fmt )
  {
    reporter r;
    r.error( file, line, fmt );
  }
  void error( const text_line& line, const boost::format& fmt );

  /*!
   *  \brief  vIG[
   *  \param  fmt   o̓bZ[W
   *  \throw  toppers::reporter::terminator
   */
  inline void fatal( const boost::format& fmt )
  {
    reporter r;
    r.fatal( fmt );
  }
  /*!
   *  \brief  vIG[isԍj
   *  \param  file  vIG[t@C
   *  \param  line  vIG[sԍ
   *  \param  fmt   o̓bZ[W
   *  \throw  toppers::reporter::terminator
   */
  inline void fatal( const std::string& file, int line, const boost::format& fmt )
  {
    reporter r;
    r.fatal( file, line, fmt );
  }
  void fatal( const text_line& line, const boost::format& fmt );

  /*!
   *  \brief  vO̒~
   *  \param  fmt   o̓bZ[W
   *  \throw  toppers::reporter::terminator
   */
  inline void stop( const boost::format& fmt )
  {
    reporter r;
    r.stop( fmt );
  }

}

/*!
 *  \brief  reporter()̋Lqȕ։邽߂̃}N
 *  \return toppers::reporter̈ꎞIuWFNg
 *  \relates toppers::reporter
 */
#define	rout	reporter()

/*!
 *  \brief  Ή\ɂ邽߂̏ϊ}N
 *  \param  msg   boost::formatɑ΂鏑
 *  \return boost::formaẗꎞIuWFNg
 *  \relates toppers::reporter
 *
 *  ɑ΂Ẵ}NKp邱ƂŁAeւ̖|󂪉\ɂȂ܂B
 *  ۂɖ|󂳂邩ǂ́A reporter::set_translator œo^֐Ɉˑ
 *  ܂B
 */
#define	_(msg)	boost::format(toppers::reporter::translate(msg))

#endif	// ! TOPPERS_REPORTER_HPP_
