/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*!
 *  \file   toppers/text.hpp
 *  \brief  eLXgf[^ǗɊւ錾`
 *
 *  ̃t@CŒ`NX
 *  \code
 *  class basic_text<Container, CharT, Traits, Allocator>;
 *  \endcode
 */
#ifndef TOPPERS_TEXT_HPP_
#define TOPPERS_TEXT_HPP_

#include "toppers/text_line.hpp"
#include "toppers/misc.hpp"
#include <iterator>
#include <vector>
#include <boost/iterator/iterator_facade.hpp>
#include <boost/format.hpp>

namespace toppers
{

  /*!
   *  \class  basic_text  text.hpp "toppers/text.hpp"
   *  \brief  eLXgf[^Ǘev[gNX
   *
   *  t@Cǂݍ񂾃eLXgf[^̊Ǘs߂̃NXłB
   *  ̃NXł́AeLXgf[^PȂ镶ƂĂł͂ȂA͌
   *  t@Cƍsԍt`ŊǗ܂B
   *
   *  \note   ̃NX̓eLXgҏW̗prz肵̂ł͂܂B
   */
  template
  <
    template <typename, class> class Container,
    typename CharT,
    class Traits = std::char_traits<CharT>,
    class Allocator = std::allocator<CharT>
  >
  class basic_text
  {
  public:
    typedef basic_text<Container, CharT, Traits, Allocator> self_t;
    typedef std::basic_string<CharT, Traits, Allocator> string_type;
    typedef ::toppers::text_line line_type;
    typedef basic_line_buf<CharT, Traits, Allocator> line_buf;

    //  ȉ͈ʓIȃReiƂ̌݊̂߂̌^`
    ////////////////////////////////////////////////////////////////////////////////
    typedef CharT value_type;
    typedef Container<line_buf, std::allocator<line_buf> > container;
    typedef value_type& reference;
    typedef const value_type& const_reference;
    typedef value_type* pointer;
    typedef const value_type* const_pointer;
    typedef Allocator allocator_type;
    typedef typename Allocator::difference_type difference_type;
    typedef typename Allocator::size_type size_type;

    /*!
     *  \class  const_iterator text.hpp "toppers/text.hpp"
     *  \brief  toppers::basic_text ̃Ce[^
     */
    class const_iterator : public boost::iterator_facade<const_iterator, CharT, boost::random_access_traversal_tag, const CharT&>
    {
      friend class boost::iterator_core_access;
    public:
      typedef boost::iterator_facade<const_iterator, CharT, boost::random_access_traversal_tag, const CharT&> base_t;
      typedef typename container::const_iterator row_iterator;
      typedef typename base_t::value_type value_type;
      typedef typename base_t::reference reference;
      typedef typename base_t::pointer pointer;
      typedef typename base_t::difference_type difference_type;

      const_iterator() : row_(), col_( 0 ) {}
      const_iterator( row_iterator row, size_type col ) : row_( row ), col_( col ) {}
      const line_type& line() const { return row_->line_; }
      const row_iterator& get_row() const { return row_; }
      size_type get_col() const { return col_; }
    protected:
      reference dereference() const { return row_->buf_[col_]; }
      bool equal( const const_iterator& rhs ) const { return row_ == rhs.row_ && col_ == rhs.col_; }
      void increment() { if ( ++col_ == row_->buf_.size() ) { col_ = 0; ++row_; } }
      void decrement() { if ( col_ == 0 ) col_ = ( --row_ )->buf_.size(); --col_; }
      void advance( difference_type n )
      {
        typename container::const_iterator row = row_;
        size_type col = col_;
        while ( col + n > row->buf_.size() )
        {
          n -= row->buf_.size() - col;
          ++row;
          col = 0;
        }
        col += n;
        if ( col == row->buf_.size() )
        {
          col = 0;
          ++row;
        }
        row_ = row;
        col_ = col;
      }
      difference_type distance_to( const const_iterator& rhs ) const
      {
        difference_type n;
        typename container::const_iterator row = row_;
        if ( rhs.row_ < row_ )
        {
          n = -static_cast<difference_type>( col_ + ( rhs.row_->buf_.size() - rhs.col_ ) );
          while ( --row != rhs.row_ )
          {
            n -= row->buf_.size();
          }
        }
        else if ( row_ < rhs.row_ )
        {
          n = col_ + rhs.col_;
          while ( ++row != rhs.row_ )
          {
            n += row->buf_.size();
          }
        }
        else
        {
          n = static_cast<difference_type>( rhs.col_ ) - static_cast<difference_type>( col_ );
        }
        return n;
      }
    private:
      typename container::const_iterator row_;
      size_type col_;
    };

    /*!
     *  \class  iterator text.hpp "toppers/text.hpp"
     *  \brief  toppers::basic_text ̃Ce[^
     */
    class iterator : public const_iterator
    {
    public:
      typedef typename container::iterator row_iterator;
      typedef typename const_iterator::value_type value_type;
      typedef typename const_iterator::reference reference;
      typedef typename const_iterator::pointer pointer;
      typedef typename const_iterator::difference_type difference_type;

      iterator() {}
      explicit iterator( row_iterator row, size_type col = 0 ) : const_iterator( row, col ) {}
      value_type& operator*() const { return const_cast<value_type&>( dereference() ); }
      iterator& operator++() { increment(); return *this; }
      iterator operator++( int ) { iterator t( *this ); increment(); return t; }
      iterator& operator--() { decrement(); return *this; }
      iterator operator--( int ) { iterator t( *this ); decrement(); return t; }
    };

    typedef std::reverse_iterator<const_iterator> const_reverse_iterator;
    typedef std::reverse_iterator<iterator> reverse_iterator;

  public:
    /*!
     *  \brief  ftHgRXgN^
     */
    basic_text() {}

    /*!
     *  \brief  RXgN^
     *  \param  first   ɗpf[^̐擪
     *  \param  last    ɗpf[^̏I[+1
     */
    basic_text( const const_iterator& first, const const_iterator& last )
    {
      if ( first != last )
      {
        init( first, last );
      }
    }

    /*!
     *  \brief  RXgN^
     *  \param  first   ɗpf[^̐擪
     *  \param  last    ɗpf[^̏I[+1
     *  \note   t@C"unknown"ɂȂ܂B
     */
    template <class ForwardIterator>
    basic_text( ForwardIterator first, ForwardIterator last )
    {
      if ( first != last )
      {
        init( first, last, append_directive );
      }
    }

    /*!
     *  \brief  RXgN^
     *  \param  first   ɗpf[^̐擪
     *  \param  last    ɗpf[^̏I[+1
     *  \param  directive PsReiɊi[邽߂̏
     *  \note   t@C"unknown"ɂȂ܂B
     *
     *  directive  void directive( container& cont, line_buf& buf )̌`
     *  Ƃ֐ւ̃|C^܂̓t@N^ł邱Ƃz肵Ă܂B
     *  cont ͓Ńf[^̊i[ɎgpReiA buf ͂Ps̕
     *  ƃt@CEsԍi[obt@łB
     */
    template <class ForwardIterator, class Directive>
    basic_text( ForwardIterator first, ForwardIterator last, Directive directive )
    {
      if ( first != last )
      {
        init( first, last, directive );
      }
    }

    /*!
     *  \brief  RXgN^
     *  \param  istr    ɗp̓Xg[
     *  \note   t@C"unknown"ɂȂ܂B
     */
    basic_text( std::basic_istream<CharT, Traits>& istr )
    {
      if ( first != last )
      {
        init( istr, append_directive );
      }
    }

    /*!
     *  \brief  RXgN^
     *  \param  istr    ɗp̓Xg[
     *  \param  directive PsReiɊi[邽߂̏
     *  \note   t@C"unknown"ɂȂ܂B
     *
     *  directive  void directive( container& cont, line_buf& buf )̌`
     *  Ƃ֐ւ̃|C^܂̓t@N^ł邱Ƃz肵Ă܂B
     *  cont ͓Ńf[^̊i[ɎgpReiA buf ͂Ps̕
     *  ƃt@CEsԍi[obt@łB
     */
    template <class Directive>
    basic_text( std::basic_istream<CharT, Traits>& istr, Directive directive )
    {
      init( istr, directive );
    }

    /*!
     *  \brief  擪ʒu̎擾
     *  \return 擪ւ̃Ce[^Ԃ
     */
    const_iterator begin() const
    {
      return const_iterator( container_.begin() );
    }

    /*!
     *  \brief  擪ʒu̎擾
     *  \return 擪ւ̃Ce[^Ԃ
     */
    iterator begin()
    {
      return iterator( container_.begin() );
    }

    /*!
     *  \brief  I[ʒu+1̎擾
     *  \return I[̎vfւ̃Ce[^Ԃ
     */
    const_iterator end() const
    {
      return const_iterator( container_.end() );
    }

    /*!
     *  \brief  I[ʒu+1̎擾
     *  \return I[̎vfւ̃Ce[^Ԃ
     */
    iterator end()
    {
      return iterator( container_.end() );
    }

    /*!
     *  \brief  t擪ʒu̎擾
     *  \return t擪ւ̋tCe[^Ԃ
     */
    const_iterator rbegin() const
    {
      const string_type* pbuf = &container_.back().buf_;
      return const_reverse_iterator( container_.rbegin(), pbuf->empty() ? 0 : pbuf->size()-1 );
    }

    /*!
     *  \brief  t擪ʒu̎擾
     *  \return t擪ւ̋tCe[^Ԃ
     */
    iterator rbegin()
    {
      string_type* pbuf = &container_.back().buf_;
      return reverse_iterator( container_.rbegin(), pbuf->empty() ? 0 : pbuf->size()-1 );
    }

    /*!
     *  \brief  tI[ʒu+1̎擾
     *  \return tI[̎vfւ̋tCe[^Ԃ
     */
    const_iterator rend() const
    {
      return const_reverse_iterator( container_.rend(), container_.back().buf_.size()-1 );
    }

    /*!
     *  \brief  tI[ʒu+1̎擾
     *  \return tI[̎vfւ̋tCe[^Ԃ
     */
    iterator rend()
    {
      return reverse_iterator( container_.rend() );
    }

    /*!
     *  \brief  擪̎Q
     *  \return 擪ւ̎QƂԂ
     */
    const_reference front() const
    {
      return container_.front().buf_[0];
    }

    /*!
     *  \brief  擪̎Q
     *  \return 擪ւ̎QƂԂ
     */
    reference front()
    {
      return container_.front().buf_[0];
    }

    /*!
     *  \brief  I[̎Q
     *  \return I[ւ̎QƂԂ
     */
    const_reference back() const
    {
      const string_type* pbuf = &container_.back().buf_;
      return pbuf_->at( pbuf_->size()-1 );
    }

    /*!
     *  \brief  I[̎Q
     *  \return I[ւ̎QƂԂ
     */
    reference back()
    {
      string_type* pbuf = &container_.back().buf_;
      return pbuf_->at( pbuf_->size()-1 );
    }

    /*!
     *  \brief  wʒu̎̕Q
     *  \return wʒu̕ւ̎QƂԂ
     */
    const_reference at( size_type pos ) const
    {
      return container_.front().buf_[pos];
    }

    /*!
     *  \brief  wʒu̎̕Q
     *  \return wʒu̕ւ̎QƂԂ
     */
    reference at( size_type pos )
    {
      return container_.front().buf_[pos];
    }

    /*!
     *  \brief  w͈͂̃eLXgf[^ǉ
     *  \param  first ǉɗpf[^̐擪
     *  \param  last  ǉɗpf[^̏I[+1
     */
    void append( const const_iterator& first, const const_iterator& last )
    {
      if ( first != last )
      {
        self_t t( *this );
        t.init( first, last );
        swap( t );
      }
    }

    /*!
     *  \brief  w͈͂̃eLXgf[^ǉ
     *  \param  first ǉɗpf[^̐擪
     *  \param  last  ǉɗpf[^̏I[+1
     *  \note   t@Cƍsԍ͒ǉ̏I[̂̂p܂B
     *          ǉɃf[^Ȃꍇ"unknown"ɂȂ܂B
     */
    template <class ForwardIterator>
    void append( ForwardIterator first, ForwardIterator last )
    {
      append( first, last, append_directive );
    }

    /*!
     *  \brief  w͈͂̃eLXgf[^ǉ
     *  \param  first ǉɗpf[^̐擪
     *  \param  last  ǉɗpf[^̏I[+1
     *  \param  directive PsReiɊi[邽߂̏
     *  \note   t@Cƍsԍ͒ǉ̏I[̂̂p܂B
     *          ǉɃf[^Ȃꍇ"unknown"ɂȂ܂B
     *
     *  directive  void directive( container& cont, line_buf& buf )̌`
     *  Ƃ֐ւ̃|C^܂̓t@N^ł邱Ƃz肵Ă܂B
     *  cont ͓Ńf[^̊i[ɎgpReiA buf ͂Ps̕
     *  ƃt@CEsԍi[obt@łB
     */
    template <class ForwardIterator, class Directive>
    void append( ForwardIterator first, ForwardIterator last, Directive directive )
    {
      if ( first != last )
      {
        self_t t( *this );
        t.init( first, last, directive );
        swap( t );
      }
    }

    /*!
     *  \brief  w͈͂̃eLXgf[^ǉ
     *  \param  istr      ǉɗp̓Xg[
     *  \note   t@Cƍsԍ͒ǉ̏I[̂̂p܂B
     *          ǉɃf[^Ȃꍇ"unknown"ɂȂ܂B
     */
    void append( std::basic_istream<CharT, Traits>& istr )
    {
      append( istr, append_directive );
    }

    /*!
     *  \brief  w͈͂̃eLXgf[^ǉ
     *  \param  istr      ǉɗp̓Xg[
     *  \param  directive PsReiɊi[邽߂̏
     *  \note   t@Cƍsԍ͒ǉ̏I[̂̂p܂B
     *          ǉɃf[^Ȃꍇ"unknown"ɂȂ܂B
     *
     *  directive  void directive( container& cont, line_buf& buf )̌`
     *  Ƃ֐ւ̃|C^܂̓t@N^ł邱Ƃz肵Ă܂B
     *  cont ͓Ńf[^̊i[ɎgpReiA buf ͂Ps̕
     *  ƃt@CEsԍi[obt@łB
     */
    template <class Directive>
    void append( std::basic_istream<CharT, Traits>& istr, Directive directive )
    {
      self_t t( *this );
      t.init( istr, directive );
      swap( t );
    }

    /*!
     *  \brief  w͈͂̃eLXgf[^
     *  \param  first ɗpf[^̐擪
     *  \param  last  ɗpf[^̏I[+1
     */
    void assign( const const_iterator& first, const const_iterator& last )
    {
      self_t t( first, last );
      swap( t );
    }

    /*!
     *  \brief  w͈͂̃eLXgf[^
     *  \param  first ɗpf[^̐擪
     *  \param  last  ɗpf[^̏I[+1
     *  \note   t@C"unknown"ɂȂ܂B
     */
    template <class ForwardIterator>
    void assign( ForwardIterator first, ForwardIterator last )
    {
      self_t t( first, last, append_directive );
      swap( t );
    }

    /*!
     *  \brief  w͈͂̃eLXgf[^
     *  \param  first ɗpf[^̐擪
     *  \param  last  ɗpf[^̏I[+1
     *  \param  directive PsReiɊi[邽߂̏
     *  \note   t@C"unknown"ɂȂ܂B
     *
     *  directive  void directive( container& cont, line_buf& buf )̌`
     *  Ƃ֐ւ̃|C^܂̓t@N^ł邱Ƃz肵Ă܂B
     *  cont ͓Ńf[^̊i[ɎgpReiA buf ͂Ps̕
     *  ƃt@CEsԍi[obt@łB
     */
    template <class ForwardIterator, class Directive>
    void assign( ForwardIterator first, ForwardIterator last, Directive directive )
    {
      self_t t( first, last, directive );
      swap( t );
    }

    /*!
     *  \brief  w͈͂̃eLXgf[^
     *  \param  istr      ɗp̓Xg[
     *  \note   t@Cƍsԍ͑̏I[̂̂p܂B
     *          ǉɃf[^Ȃꍇ"unknown"ɂȂ܂B
     */
    void assign( std::basic_istream<CharT, Traits>& istr )
    {
      self_t t( istr, append_directive );
      swap( t );
    }

    /*!
     *  \brief  w͈͂̃eLXgf[^
     *  \param  istr      ɗp̓Xg[
     *  \param  directive PsReiɊi[邽߂̏
     *  \note   t@Cƍsԍ͑̏I[̂̂p܂B
     *          ǉɃf[^Ȃꍇ"unknown"ɂȂ܂B
     *
     *  directive  void directive( container& cont, line_buf& buf )̌`
     *  Ƃ֐ւ̃|C^܂̓t@N^ł邱Ƃz肵Ă܂B
     *  cont ͓Ńf[^̊i[ɎgpReiA buf ͂Ps̕
     *  ƃt@CEsԍi[obt@łB
     */
    template <class Directive>
    void assign( std::basic_istream<CharT, Traits>& istr, Directive directive )
    {
      self_t t( istr, directive );
      swap( t );
    }

    /*!
     *  \brief  I[ɕǉ
     *  \param  value ǉ镶
     */
    void push_back( value_type value )
    {
      if ( container_.empty() )
      {
        line_buf buf( initial_line(), string_type( 1, value ) );
        container_.push_back( buf );
      }
      else
      {
        const string_type* pbuf = &container_.back().buf_;
        if ( pbuf_->at( pbuf_->size()-1 ) == widen( '\n' ) )
        {
          line_buf buf( initial_line(), string_type( 1, value ) );
          ++buf.line_.line_;
          container_.push_back( buf );
        }
        else
        {
          container_.back().buf_ += value;
        }
      }
    }

    /*!
     *  \brief  I[當폜
     */
    void pop_back()
    {
      if ( container_.empty() )
      {
        container_.pop_back();
      }
      else
      {
        const string_type* pbuf = &container_.back().buf_;
        pbuf->resize( pbuf->size()-1 );
        if ( container_.back().buf_.empty() )
        {
          container_.pop_back();
        }
      }
    }

    /*!
     *  \brief  Rei󂩂ǂ̔
     *  \return ̏ꍇ true Ԃ
     */
    bool empty() const
    {
      return container_.empty();
    }

    /*!
     *  \brief  ReĩTCY
     *  \return ReiɊ܂܂Ă镶Ԃ
     */
    size_type size() const
    {
      return std::accumulate( container_.begin(), container_.end(), size_type( 0 ), add_line_size );
    }

    /*!
     *  \brief  Reǐ
     *  \param  other   ΏۂƂȂRei
     */
    void swap( self_t& other )
    {
      container_.swap( other.container_ );
      line_.swap( other.line_ );
    }

    /*!
     *  \brief  ReĩNA
     */
    void clear()
    {
      container_.clear();
    }
  private:
    //! ̃Rei͌Ƃ鏉
    void init( const const_iterator& first, const const_iterator& last )
    {
      line_buf buf( *first.get_row() );
      if ( first.get_col() > 0 )
      {
        buf.buf_ = buf.buf_.substr( first.get_col() );
      }
      container_.push_back( buf );
      ++first;
      container_.append( first.get_row(), last.get_row() );
      buf = *last.get_row();
      buf.buf_ = buf.buf_.substr( 0, last.get_col() );
      container_.push_back( buf );
    }

    //! َ̃Rei͌Ƃ鏉
    template <class ForwardIterator, class Directive>
    void init( ForwardIterator first, ForwardIterator last, Directive directive )
    {
      line_buf buf( initial_line() );
      const value_type nl = widen( '\n' );
      for ( ForwardIterator iter = first; iter != last; ++iter )
      {
        value_type ch = *iter;
        buf.buf_ += ch;
        if ( ch == nl )
        {
          directive( container_, buf );
          line_ = buf.line_;
        }
      }
      if ( !buf.buf_.empty() )
      {
        container_.push_back( buf );
      }
    }

    //! ̓Xg[͌Ƃ鏉
    template <class Directive>
    void init( std::basic_istream<CharT, Traits>& istr, Directive directive )
    {
      line_buf buf( initial_line() );
      const value_type nl = widen( '\n' );
      while ( istr )
      {
        value_type ch;
        istr.get( ch );
        buf.buf_ += ch;
        if ( ch == nl )
        {
          directive( container_, buf );
          line_ = buf.line_;
        }
      }
      if ( !buf.buf_.empty() )
      {
        container_.push_back( buf );
      }
    }

    //! t@CEsԍ̏̎擾
    const line_type initial_line() const
    {
      if ( line_.line_ > 0 )
      {
        return line_;
      }
      if ( container_.empty() )
      {
        return line_type( "unknown", 1 );
      }
      return container_.back().line_;
    }

    //! VOoCgʌ`̕^ւ̕ϊ
    static value_type widen( char ch )
    {
      return ::toppers::widen<value_type>( ch );
    }

    //! size ֐ŎgpݎZ
    static size_type add_line_size( size_type value, const line_buf& buf )
    {
      return value + buf.buf_.size();
    }

    //! Directive ̃ftHg
    static void append_directive( container& cont, line_buf& buf )
    {
      cont.push_back( buf );
      ++buf.line_.line_;
      buf.buf_.clear();
    }
  private:
    container container_;
    line_type line_;
  };

  typedef basic_text<std::vector, char> text;

  inline const text_line& get_text_line( text::const_iterator iter )
  {
    return iter.line();
  }

  inline const text_line& get_text_line( text::iterator iter )
  {
    return iter.line();
  }

}

#endif  // ! TOPPERS_TEXT_HPP_
