/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 *
 *  Copyright (C) 2005 by Takagi Nobuhisa
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 */

/*!
 *  \file   toppers/text_line.hpp
 *  \brief  eLXgf[^̍sɊւ錾`
 *
 *  ̃t@CŒ`NX
 *  \code
 *  struct text_line;
 *  struct basic_line_buf<CharT, Traits, Allocator>;
 *  \endcode
 */
#ifndef TOPPERS_TEXTLINE_HPP_
#define TOPPERS_TEXTLINE_HPP_

#include "toppers/config.hpp"
#include <iosfwd>
#include <string>
#include <boost/filesystem/path.hpp>

namespace toppers
{

  /*!
   *  \struct text_line text_line.hpp "toppers/text_line.hpp"
   *  \brief  t@Cƍsԍ̕ێNX
   */
  struct text_line
  {
    boost::filesystem::path file_;  //!< t@C
    long line_; //!< sԍi1`j

    /*!
     *  \brief  ftHgRXgN^
     */
    text_line()
      : line_( 0 )
    {
    }
    /*!
     *  \brief  RXgN^
     *  \param  file  t@C
     *  \param  line  sԍ
     */
    explicit text_line( const boost::filesystem::path& file, long line = 0 )
      : file_( file ), line_( line )
    {
    }
    /*!
     *  \brief  RXgN^
     *  \param  file  t@C
     *  \param  line  sԍ
     */
    explicit text_line( const std::string& file, long line = 0 )
      : file_( file, boost::filesystem::native ), line_( line )
    {
    }
    /*!
     *  \brief  RXgN^
     *  \param  file  t@C
     *  \param  line  sԍ
     */
    explicit text_line( const char* file, long line = 0 )
      : file_( file, boost::filesystem::native ), line_( line )
    {
    }
    /*!
     *  \brief  IuWFNǧ
     *  \param  other   Ώۂ̃IuWFNg
     */
    void swap( text_line& other ) throw()
    {
      long t( line_ );
      line_ = other.line_;
      other.line_ = t;
      const_cast<std::string&>( file_.string() ).swap( const_cast<std::string&>( other.file_.string() ) );
    }
  };

  /*!
   *  \struct line_buf text_line.hpp "toppers/text_line.hpp"
   *  \brief  1seLXg̊ǗNX
   */
  template
  <
    typename CharT,
    class Traits = std::char_traits<CharT>,
    class Allocator = std::allocator<CharT>
  >
  struct basic_line_buf
  {
    typedef std::basic_string<CharT, Traits, Allocator> string_type;

    string_type buf_; //!< sf[^\
    text_line line_;  //!< sԍ

    /*!
     *  \brief  ftHgRXgN^
     */
    basic_line_buf()
    {
    }
    /*!
     *  \brief  RXgN^
     *  \param  line  sԍ
     */
    basic_line_buf( const text_line& line )
      : line_( line )
    {
    }
    /*!
     *  \brief  RXgN^
     *  \param  line  sԍ
     *  \param  data  sf[^
     */
    basic_line_buf( const text_line& line, const string_type& data )
      : buf_( data ), line_( line )
    {
    }
  };

  typedef basic_line_buf<char> line_buf;

  /*!
   *  \brief  Ce[^ێsԍ̎擾
   *  \param  iter  Ce[^
   *  \return sԍւ̎QƂԂ
   *
   *  iter Ŏw肵Ce[^sԍێĂȂA̍sԍԂ܂B
   *  sԍێĂȂꍇ̓_~[IuWFNgւ̎QƂԂ܂B
   */
  template <class Iterator>
  inline const text_line& get_text_line( Iterator iter )
  {
    static text_line dummy;
    return dummy;
  }

}

#endif  // ! TOPPERS_TEXTLINE_HPP_
