/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 * 
 *  Copyright (C) 2000-2004 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2005-2006 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN  
 * 
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation 
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 * 
 *  @(#) $Id: sys_manage.c,v 1.5 2006/03/21 09:01:57 honda Exp $
 */

/** \file
 *	ƥִǽ
 */

#include "fdmp_kernel.h"
#include "check.h"
#include "task.h"


/**
 *  ̤ͥβž
 */
#ifdef __rot_rdq

SYSCALL ER
rot_rdq(PRI tskpri)
{
	UINT	pri;
	ER		ercd;
	BOOL	dspreq = FALSE;

	LOG_ROT_RDQ_ENTER(tskpri);
	CHECK_TSKCTX_UNL();
	CHECK_TPRI_SELF(tskpri);

  retry:	
	T_ACQUIRE_LOCK(runccb->tsk_lock);
	pri = (tskpri == TPRI_SELF) ? runtsk->priority : INT_PRIORITY(tskpri);
	dspreq = rotate_ready_queue(my_local_ccb, pri);
	ercd = E_OK;
	T_RELEASE_LOCK_AND_DISPATCH(runccb->tsk_lock, dspreq);

	exit:
	LOG_ROT_RDQ_LEAVE(ercd);
	return(ercd);
}

#endif /* __rot_rdq */

/**
 *  ̤ͥβž󥿥ƥѡ
 */
#ifdef __irot_rdq

SYSCALL ER
irot_rdq(PRI tskpri)
{
	ER	ercd;

	LOG_IROT_RDQ_ENTER(tskpri);
	CHECK_INTCTX_UNL();
	CHECK_TPRI(tskpri);

  retry:		
	I_ACQUIRE_LOCK(runccb->tsk_lock);
	if (rotate_ready_queue(my_local_ccb, INT_PRIORITY(tskpri))) {
		reqflg = TRUE;
	}
	ercd = E_OK;
	I_RELEASE_LOCK(runccb->tsk_lock)

	exit:
	LOG_IROT_RDQ_LEAVE(ercd);
	return(ercd);
}

#endif /* __irot_rdq */

/**
 *  ̤ͥβžʥޥץåѡ
 */
#ifdef __mrot_rdq

SYSCALL ER
mrot_rdq(ID clsid, PRI tskpri)
{
	UINT	pri;
	CCB		*ccb;
	BOOL	dspreq = FALSE;
	ER 		ercd;

	LOG_ROT_RDQ_ENTER(tskpri);
	CHECK_TSKCTX_UNL();
	CHECK_TPRI(tskpri);
	CHECK_CLSID_SELF(clsid);
	ccb = t_get_ccb_self(clsid);

  retry:
	T_ACQUIRE_LOCK(ccb->tsk_lock);
	pri = INT_PRIORITY(tskpri);
	if (rotate_ready_queue(ccb, pri)) {
		dspreq = dispatch_request(ccb->prcid);
	}
	T_RELEASE_LOCK_AND_DISPATCH(ccb->tsk_lock, dspreq);

	exit:
	LOG_ROT_RDQ_LEAVE(ercd);
	return(E_OK);
}

#endif /* __mrot_rdq */

/**
 *  ̤ͥβžʥޥץå/󥿥ƥѡ
 */
#ifdef __imrot_rdq

SYSCALL ER
imrot_rdq(ID clsid, PRI tskpri)
{
	CCB		*ccb;
	ER		ercd;

	LOG_IROT_RDQ_ENTER(tskpri);
	CHECK_INTCTX_UNL();
	CHECK_TPRI(tskpri);
	ccb = i_get_ccb_self(clsid);

  retry:		
	I_ACQUIRE_LOCK(ccb->tsk_lock);
	if (rotate_ready_queue(ccb, INT_PRIORITY(tskpri))) {
		reqflg = dispatch_request(ccb->prcid) ? TRUE : reqflg;
	}
	ercd = E_OK;
	I_RELEASE_LOCK(ccb->tsk_lock)

	exit:
	LOG_IROT_RDQ_LEAVE(ercd);
	return(ercd);
}

#endif /* __imrot_rdq */

/**
 *  ¹Ծ֤ΥIDλ
 */
#ifdef __get_tid

SYSCALL ER
get_tid(ID *p_tskid)
{
	ER	ercd;

	LOG_GET_TID_ENTER(p_tskid);
	CHECK_TSKCTX_UNL();

	t_lock_cpu();
	*p_tskid = TSKID(runtsk);
	ercd = E_OK;
	t_unlock_cpu();

	exit:
	LOG_GET_TID_LEAVE(ercd, *p_tskid);
	return(ercd);
}

#endif /* __get_tid */

/**
 *  ¹Ծ֤ΥIDλȡ󥿥ƥѡ
 */
#ifdef __iget_tid

SYSCALL ER
iget_tid(ID *p_tskid)
{
	ER	ercd;

	LOG_IGET_TID_ENTER(p_tskid);
	CHECK_INTCTX_UNL();

	i_lock_cpu();
	*p_tskid = (runtsk == NULL) ? TSK_NONE : TSKID(runtsk);
	ercd = E_OK;
	i_unlock_cpu();

	exit:
	LOG_IGET_TID_LEAVE(ercd, *p_tskid);
	return(ercd);
}

#endif /* __iget_tid */

/**
 *  ¹Ծ֤ΥIDλȡʥޥץåѡ
 */
#ifdef __mget_tid

SYSCALL ER
mget_tid(ID clsid, ID *p_tskid)
{
	ER	ercd;
	CCB		*ccb;
	
	CHECK_TSKCTX_UNL();
	CHECK_CLSID_SELF(clsid);
	ccb = t_get_ccb_self(clsid);

	t_lock_cpu();
	*p_tskid = ((*ccb->p_runtsk) == NULL) ? TSK_NONE : MTSKID((ccb->tsk.cb_table), (*ccb->p_runtsk));
	ercd = E_OK;
	t_unlock_cpu();
	
	exit:
	return(ercd);
}

#endif /* __mget_tid */

/**
 *  ¹Ծ֤ΥIDλȡʥޥץå/󥿥ƥѡ
 */
#ifdef __imget_tid

SYSCALL ER
imget_tid(ID clsid, ID *p_tskid)
{
	ER	ercd;
	CCB		*ccb;

	CHECK_INTCTX_UNL();
	
	CHECK_CLSID_SELF(clsid);
	ccb = i_get_ccb_self(clsid);

	i_lock_cpu();
	*p_tskid = ((*ccb->p_runtsk) == NULL) ? TSK_NONE : MTSKID((ccb->tsk.cb_table), (*ccb->p_runtsk));
	ercd = E_OK;
	i_unlock_cpu();

	exit:
	return(ercd);
}

#endif /* __imget_tid */

/**
 *  CPUå֤ؤΰܹ
 */
#ifdef __loc_cpu

SYSCALL ER
loc_cpu(void)
{
	ER	ercd;

	LOG_LOC_CPU_ENTER();
	CHECK_TSKCTX();

	if (!(t_sense_lock())) {
		t_lock_cpu();
	}
	ercd = E_OK;

	exit:
	LOG_LOC_CPU_LEAVE(ercd);
	return(ercd);
}

#endif /* __loc_cpu */

/**
 *  CPUå֤ؤΰܹԡ󥿥ƥѡ
 */
#ifdef __iloc_cpu

SYSCALL ER
iloc_cpu(void)
{
	ER	ercd;

	LOG_ILOC_CPU_ENTER();
	CHECK_INTCTX();

	if (!(i_sense_lock())) {
		i_lock_cpu();
	}
	ercd = E_OK;

	exit:
	LOG_ILOC_CPU_LEAVE(ercd);
	return(ercd);
}

#endif /* __iloc_cpu */

/**
 *  CPUå֤β
 *
 *  CPUåϡǥѥåɬפȤʤ륵ӥƤӽФ
 *  ȤϤǤʤᡤCPUå֤β˥ǥѥåư
 *  ɬפϤʤ
 */
#ifdef __unl_cpu

SYSCALL ER
unl_cpu(void)
{
	ER	ercd;

	LOG_UNL_CPU_ENTER();
	CHECK_TSKCTX();

	if (t_sense_lock()) {
		t_unlock_cpu();
	}
	ercd = E_OK;

	exit:
	LOG_UNL_CPU_LEAVE(ercd);
	return(ercd);
}

#endif /* __unl_cpu */

/**
 *  CPUå֤β󥿥ƥѡ
 *
 *  CPUåϡǥѥåɬפȤʤ륵ӥƤӽФ
 *  ȤϤǤʤᡤCPUå֤β˥ǥѥåεư
 *  ᤹ɬפϤʤ
 */
#ifdef __iunl_cpu

SYSCALL ER
iunl_cpu(void)
{
	ER	ercd;

	LOG_IUNL_CPU_ENTER();
	CHECK_INTCTX();

	if (i_sense_lock()) {
		i_unlock_cpu();
	}
	ercd = E_OK;

	exit:
	LOG_IUNL_CPU_LEAVE(ercd);
	return(ercd);
}

#endif /* __iunl_cpu */

/**
 *  ǥѥåζػ
 */
#ifdef __dis_dsp

SYSCALL ER
dis_dsp(void)
{
	ER	ercd;

	LOG_DIS_DSP_ENTER();
	CHECK_TSKCTX_UNL();

  retry:		
	T_ACQUIRE_LOCK_AND_CHECK_RUNNABLE();
	enadsp = FALSE;
	ercd = E_OK;
	T_RELEASE_LOCK(runccb->tsk_lock);

	exit:
	LOG_DIS_DSP_LEAVE(ercd);
	return(ercd);
}

#endif /* __dis_dsp */

/**
 *  ǥѥåε
 */
#ifdef __ena_dsp

SYSCALL ER
ena_dsp(void)
{
	ER	ercd;
	BOOL dspreq = FALSE;

	LOG_ENA_DSP_ENTER();
	CHECK_TSKCTX_UNL();
	
  retry:
	T_ACQUIRE_LOCK(runccb->tsk_lock);
	enadsp = TRUE;
	if (runtsk->terflg) {
		T_RELEASE_LOCK(runccb->tsk_lock);
		exit_and_dispatch();
	}
	if (runtsk != schedtsk) {
		dspreq = TRUE;
	}
	T_RELEASE_LOCK_AND_DISPATCH(runccb->tsk_lock, dspreq);
	ercd = E_OK;

  exit:
	LOG_ENA_DSP_LEAVE(ercd);
	return(ercd);
}

#endif /* __ena_dsp */

/**
 *  ƥȤλ
 */
#ifdef __sns_ctx

SYSCALL BOOL
sns_ctx(void)
{
	BOOL	state;

	LOG_SNS_CTX_ENTER();
	state = sense_context() ? TRUE : FALSE;
	LOG_SNS_CTX_LEAVE(state);
	return(state);
}

#endif /* __sns_ctx */

/**
 *  CPUå֤λ
 */
#ifdef __sns_loc

SYSCALL BOOL
sns_loc(void)
{
	BOOL	state;

	LOG_SNS_LOC_ENTER();
	state = sense_lock() ? TRUE : FALSE;
	LOG_SNS_LOC_LEAVE(state);
	return(state);
}

#endif /* __sns_loc */

/**
 *  ǥѥåػ߾֤λ
 */
#ifdef __sns_dsp

SYSCALL BOOL
sns_dsp(void)
{
	BOOL	state;

	LOG_SNS_DSP_ENTER();
	state = !(enadsp) ? TRUE : FALSE;
	LOG_SNS_DSP_LEAVE(state);
	return(state);
}

#endif /* __sns_dsp */

/**
 *  ǥѥåα֤λ
 */
#ifdef __sns_dpn

SYSCALL BOOL
sns_dpn(void)
{
	BOOL	state;

	LOG_SNS_DPN_ENTER();
	state = (sense_context() || sense_lock() || !(enadsp)) ? TRUE : FALSE;
	LOG_SNS_DPN_LEAVE(state);
	return(state);
}

#endif /* __sns_dpn */

/**
 *  ͥư֤λ
 */
#ifdef __vsns_ini

SYSCALL BOOL
vsns_ini(void)
{
	BOOL	state;

	LOG_VSNS_INI_ENTER();
	state = !(iniflg) ? TRUE : FALSE;
	LOG_VSNS_INI_LEAVE(state);
	return(state);
}

#endif /* __vsns_ini */
