/*
 *  TOPPERS/FDMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Function Distributed Multiprocessor Kernel
 * 
 *  Copyright (C) 2000-2004 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2005-2006 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 * 
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation 
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 * 
 *  @(#) $Id: task_sync.c,v 1.7 2007/06/05 01:10:01 honda Exp $
 */

/** \file
 *	°Ʊǽ
 *
 */

#include "fdmp_kernel.h"
#include "check.h"
#include "class.h"
#include "task.h"
#include "wait.h"

/**
 *  Ԥ
 */
#ifdef __slp_tsk

SYSCALL ER
slp_tsk()
{
	WINFO	winfo;
	ER		ercd;

	LOG_SLP_TSK_ENTER();
	CHECK_DISPATCH();

  retry:
	T_ACQUIRE_LOCK_AND_CHECK_RUNNABLE();
	if (runtsk->wupcnt) {
		runtsk->wupcnt = FALSE;
		RELEASE_LOCK(runccb->tsk_lock);
		ercd = E_OK;
	}
	else {
		runtsk->tstat = (TS_WAITING | TS_WAIT_SLEEP);
		make_wait(&winfo);
		LOG_TSKSTAT(runtsk);
		RELEASE_LOCK(runccb->tsk_lock);
		dispatch();
		ercd = winfo.wercd;
	}
	t_unlock_cpu();

  exit:
	LOG_SLP_TSK_LEAVE(ercd);
	return(ercd);
}

#endif /* __slp_tsk */

/**
 *  ԤʥॢȤ
 */
#ifdef __tslp_tsk

SYSCALL ER
tslp_tsk(TMO tmout)
{
	WINFO	winfo;
	TMEVTB	tmevtb;
	ER		ercd;
	
	LOG_TSLP_TSK_ENTER(tmout);
	CHECK_DISPATCH();
	CHECK_TMOUT(tmout);
	
  retry:
	T_ACQUIRE_LOCK_AND_CHECK_RUNNABLE();
	if (runtsk->wupcnt) {
		runtsk->wupcnt = FALSE;
		RELEASE_LOCK(runccb->tsk_lock);
		ercd = E_OK;
	}
	else if (tmout == TMO_POL) {
		RELEASE_LOCK(runccb->tsk_lock);
		ercd = E_TMOUT;		
	}
	else {
		runtsk->tstat = (TS_WAITING | TS_WAIT_SLEEP);
		make_wait_tmout(&winfo, &tmevtb, tmout);
		LOG_TSKSTAT(runtsk);
		RELEASE_LOCK(runccb->tsk_lock);	
		dispatch();
		ercd = winfo.wercd;
	}
	t_unlock_cpu();

  exit:
	LOG_TSLP_TSK_LEAVE(ercd);
	return(ercd);
}

#endif /* __tslp_tsk */

/**
 *  ε
 */
#ifdef __wup_tsk

SYSCALL ER
wup_tsk(ID tskid)
{
	TCB		*tcb;
	UINT	tstat;
	ER		ercd;
	CCB 	*ccb;
	BOOL	dspreq = FALSE;

	LOG_WUP_TSK_ENTER(tskid);
	CHECK_TSKCTX_UNL();
	CHECK_CLSID_SELF(get_clsid(tskid)); 
	ccb = t_get_ccb_self(get_clsid(tskid));    
	tskid = remove_clsid(tskid);
	CHECK_TSKID_SELF(ccb, tskid);
	tcb = get_tcb_self(ccb, tskid);

  retry:
	T_ACQUIRE_LOCK(ccb->tsk_lock);
	if (TSTAT_DORMANT(tstat = tcb->tstat) || (tcb == runtsk && tcb->terflg)) {
		ercd = E_OBJ;
	}
	else if ((tstat & TS_WAIT_SLEEP) != 0) {
		if (wait_complete(ccb, tcb)) {
   			dspreq = dispatch_request(ccb->prcid);
		}
		ercd = E_OK;
	}
	else if (!(tcb->wupcnt)) {
		tcb->wupcnt = TRUE;
		ercd = E_OK;
	}
	else {
		ercd = E_QOVR;
	}
	T_RELEASE_LOCK_AND_DISPATCH(ccb->tsk_lock, dspreq);
  exit:
	LOG_WUP_TSK_LEAVE(ercd);
	return(ercd);
}

#endif /* __wup_tsk */

/**
 *  ε󥿥ƥѡ
 */
#ifdef __iwup_tsk

SYSCALL ER
iwup_tsk(ID tskid)
{
	TCB		*tcb;
	UINT	tstat;
	ER		ercd;
	CCB 	*ccb;
	
	LOG_IWUP_TSK_ENTER(tskid);
	CHECK_INTCTX_UNL();
	CHECK_CLSID_SELF(get_clsid(tskid)); 
	ccb = i_get_ccb_self(get_clsid(tskid));
	tskid = remove_clsid(tskid);
	CHECK_TSKID(ccb, tskid);
	tcb = get_tcb(ccb, tskid);
	
  retry:
	I_ACQUIRE_LOCK(ccb->tsk_lock);
	if (TSTAT_DORMANT(tstat = tcb->tstat)) {
		ercd = E_OBJ;
	}
	else if ((tstat & TS_WAIT_SLEEP) != 0) {
		if (wait_complete(ccb, tcb)) {
			if (dispatch_request(ccb->prcid)) {
				reqflg = TRUE;
			}
		}
		ercd = E_OK;
	}
	else if (!(tcb->wupcnt)) {
		tcb->wupcnt = TRUE;
		ercd = E_OK;
	}
	else {
		ercd = E_QOVR;
	}
	I_RELEASE_LOCK(ccb->tsk_lock);

  exit:
	LOG_IWUP_TSK_LEAVE(ercd);
	return(ercd);
}

#endif /* __iwup_tsk */

/**
 *  ׵Υ󥻥
 */
#ifdef __can_wup

SYSCALL ER_UINT
can_wup(ID tskid)
{
	TCB	*tcb;
	CCB	*ccb;
	ER_UINT	ercd;

	LOG_CAN_WUP_ENTER(tskid);
	CHECK_TSKCTX_UNL();
	CHECK_CLSID_SELF(get_clsid(tskid)); 
	ccb = t_get_ccb_self(get_clsid(tskid));    
	tskid = remove_clsid(tskid);
	CHECK_TSKID_SELF(ccb, tskid);
	tcb = get_tcb_self(ccb, tskid);
	
  retry:
	T_ACQUIRE_LOCK(ccb->tsk_lock);
	if (TSTAT_DORMANT(tcb->tstat) || (tcb == runtsk && tcb->terflg)) {
		ercd = E_OBJ;
	}
	else {
		ercd = tcb->wupcnt ? 1 : 0;
		tcb->wupcnt = FALSE;
	}
	T_RELEASE_LOCK(ccb->tsk_lock);

  exit:
	LOG_CAN_WUP_LEAVE(ercd);
	return(ercd);
}

#endif /* __can_wup */

/**
 *  Ԥ֤ζ
 */
#ifdef __rel_wai

SYSCALL ER
rel_wai(ID tskid)
{
	TCB		*tcb;
	ER		ercd;
	CCB		*ccb, *occb;
	BOOL	dspreq = FALSE;

	LOG_REL_WAI_ENTER(tskid);
	CHECK_TSKCTX_UNL();
	CHECK_CLSID_SELF(get_clsid(tskid)); 
	ccb = t_get_ccb_self(get_clsid(tskid));    
	tskid = remove_clsid(tskid);
	CHECK_TSKID(ccb, tskid);
	tcb = get_tcb(ccb, tskid);
	
  retry:	
	T_ACQUIRE_LOCK(ccb->tsk_lock);
	if (!(TSTAT_WAITING(tcb->tstat))) {
		ercd = E_OBJ;
	}
	else {
		if ((tcb->tstat & TS_WAIT_WOBJ) == 0) {
			dspreq = wait_release(ccb, tcb);
		}
		else {
			/*
			 * ֥Ԥξ
			 * ǥåɥåΤᡤåľ
			 */			
            occb = get_ccb(((WINFO_WOBJ *)(tcb->winfo))->clsid);
			tcb->pend_relwai = TRUE;
			RELEASE_LOCK(ccb->tsk_lock);

			/*
			 * 餿
			 *   ֥ȥå -> å
			 * νǥå
			 */
			if (ACQUIRE_LOCK(occb->obj_lock)) {
				t_unlock_cpu();
				goto retry;
			}			
			T_ACQUIRE_NESTED_LOCK(ccb->tsk_lock, occb->obj_lock);
			
			/* ξ֤ѲƤʤå */
			if (!(tcb->pend_relwai)) {
				/* ¾βսԤʤ줿 */
				T_RELEASE_NESTED_LOCK(ccb->tsk_lock, occb->obj_lock);
				T_RELEASE_LOCK(occb->obj_lock);
				ercd = E_OBJ;
				goto exit;
			}
			tcb->pend_relwai = FALSE;
			dspreq = wait_release(ccb, tcb);
			RELEASE_LOCK(occb->obj_lock);
		}
		if (dspreq){
			dspreq = dispatch_request(ccb->prcid);
		}		
		ercd = E_OK;
	}
	T_RELEASE_LOCK_AND_DISPATCH(ccb->tsk_lock, dspreq);

  exit:
	LOG_REL_WAI_LEAVE(ercd);
	return(ercd);
}

#endif /* __rel_wai */

/**
 *  Ԥ֤ζ󥿥ƥѡ
 */
#ifdef __irel_wai

SYSCALL ER
irel_wai(ID tskid)
{
	TCB		*tcb;
	ER		ercd;
	CCB		*ccb, *occb;
	BOOL	dspreq = FALSE;
	
	LOG_IREL_WAI_ENTER(tskid);
	CHECK_INTCTX_UNL();
	CHECK_CLSID_SELF(get_clsid(tskid)); 
	ccb = i_get_ccb_self(get_clsid(tskid));    
	tskid = remove_clsid(tskid);
	CHECK_TSKID(ccb, tskid);
	tcb = get_tcb(ccb, tskid);

  retry:
	I_ACQUIRE_LOCK(ccb->tsk_lock);
	if (!(TSTAT_WAITING(tcb->tstat))) {
		ercd = E_OBJ;
	}
	else {
		if ((tcb->tstat & TS_WAIT_WOBJ) == 0) {
			dspreq = wait_release(ccb, tcb);
		}
		else {
			/*
			 * ֥Ԥξ
			 * ǥåɥåΤᡤåľ
			 */			
            occb = get_ccb(((WINFO_WOBJ *)(tcb->winfo))->clsid);
			tcb->pend_relwai = TRUE;
			RELEASE_LOCK(ccb->tsk_lock);

			if (ACQUIRE_LOCK(occb->obj_lock)) {
				i_unlock_cpu();
				goto retry;
			}			
			I_ACQUIRE_NESTED_LOCK(ccb->tsk_lock, occb->obj_lock);

			/*  ֥å  */
			if (!(tcb->pend_relwai)) {
				I_RELEASE_NESTED_LOCK(ccb->tsk_lock, occb->obj_lock);
				I_RELEASE_LOCK(occb->obj_lock);
				ercd = E_OBJ;
				goto exit;
			}
			tcb->pend_relwai = FALSE;
			dspreq = wait_release(ccb, tcb);
			RELEASE_LOCK(occb->obj_lock);
		}
		if (dspreq){
			reqflg = TRUE;
		}
		ercd = E_OK;
	}
	I_RELEASE_LOCK(ccb->tsk_lock);
	
  exit:
	LOG_IREL_WAI_LEAVE(ercd);
	return(ercd);
}

#endif /* __irel_wai */

/**
 *  Ԥ֤ؤΰܹ
 * 
 *  оݤǤʤΥץåǥѥåػǤ
 *  ȯԤ롥
 *  
 */
#ifdef __sus_tsk

SYSCALL ER
sus_tsk(ID tskid)
{
	TCB		*tcb;
	UINT	tstat;
	ER		ercd;
	CCB		*ccb;
	BOOL	dspreq = FALSE;
	
	LOG_SUS_TSK_ENTER(tskid);
	CHECK_TSKCTX_UNL();
	CHECK_CLSID_SELF(get_clsid(tskid)); 
	ccb = t_get_ccb_self(get_clsid(tskid));    
	tskid = remove_clsid(tskid);
	CHECK_TSKID_SELF(ccb, tskid);
	tcb = get_tcb_self(ccb, tskid);

    /*
     * ʬȤ򥵥ڥɤϥå塤֤ѹ
     * Ƥʤåɬפ롥
     * оݤʳξϥåɬפϤʤ
     */
  retry:
    if (tcb == runtsk) {
        /* ξϥå塤֤ѹƤʤå */
        T_ACQUIRE_LOCK_AND_CHECK_RUNNABLE();
    } else {
        T_ACQUIRE_LOCK(ccb->tsk_lock);        
    }
	if (tcb == runtsk && !(enadsp)) {
		/* ʬǥѥåػ߾֤sus_tskȯԤȥ顼 */
		ercd = E_CTX;
	}
	else if (TSTAT_DORMANT(tstat = tcb->tstat)) {
		ercd = E_OBJ;
	}
	else if (TSTAT_RUNNABLE(tstat)) {
		/*
		 *  ¹ԤǤ֤鶯Ԥ֤ؤ
		 */
		tcb->tstat = TS_SUSPENDED;
		LOG_TSKSTAT(tcb);
		if (make_non_runnable(ccb, tcb)) {
			dspreq = dispatch_request(ccb->prcid);
		}
		ercd = E_OK;
	}
	else if (TSTAT_SUSPENDED(tstat)) {
		ercd = E_QOVR;
	}
	else {
		/*
		 *  Ԥ֤Ԥ֤ؤ
		 */
		tcb->tstat |= TS_SUSPENDED;
		LOG_TSKSTAT(tcb);
		ercd = E_OK;
	}
	T_RELEASE_LOCK_AND_DISPATCH(ccb->tsk_lock, dspreq);

  exit:
	LOG_SUS_TSK_LEAVE(ercd);
	return(ercd);
}

#endif /* __sus_tsk */

/**
 *  Ԥ֤κƳ
 */
#ifdef __rsm_tsk

SYSCALL ER
rsm_tsk(ID tskid)
{
	TCB		*tcb;
	UINT	tstat;
	ER		ercd;
	CCB		*ccb;
	BOOL	dspreq = FALSE;

	LOG_RSM_TSK_ENTER(tskid);
	CHECK_TSKCTX_UNL();
	CHECK_CLSID_SELF(get_clsid(tskid)); 
	ccb = t_get_ccb_self(get_clsid(tskid));    
	tskid = remove_clsid(tskid);
	CHECK_TSKID(ccb, tskid);
	tcb = get_tcb(ccb, tskid);

  retry:	
	T_ACQUIRE_LOCK(ccb->tsk_lock);
	if (!(TSTAT_SUSPENDED(tstat = tcb->tstat)) || (tcb == runtsk && tcb->terflg)) {
		ercd = E_OBJ;
	}
	else if (!(TSTAT_WAITING(tstat))) {
		/*
		 *  Ԥ֤¹ԤǤ֤ؤ
		 */
		if (make_runnable(ccb, tcb)) {
			dspreq = dispatch_request(ccb->prcid);
		}
		ercd = E_OK;
	}
	else {
		/*
		 *  Ԥ֤Ԥ֤ؤ
		 */
		tcb->tstat &= ~TS_SUSPENDED;
		LOG_TSKSTAT(tcb);
		ercd = E_OK;
	}
	T_RELEASE_LOCK_AND_DISPATCH(ccb->tsk_lock, dspreq);

  exit:
	LOG_RSM_TSK_LEAVE(ercd);
	return(ercd);
}

#endif /* __rsm_tsk */

/**
 *  Ԥ֤ζƳ
 *
 *  FDMPͥǤϡfrsm_tsk  rsm_tsk ƱνȤʤ롥frsm_tsk 
 *  ƤФȡfrsm_tsk  rsm_tsk ξΥӥΥȥ졼
 *  Ϥ롥 rsm_tsk Υȥ졼뤳
 *  ȤɬפǤ롥rsm_tsk Υȥ졼뤿ˤϡ
 *  ǥѥåΥȡ㳰Υ뤳Ȥɬ
 *  ȤʤΤǡդɬפǤ롥
 */
#ifdef __frsm_tsk

SYSCALL ER
frsm_tsk(ID tskid)
{
	ER	ercd;

	LOG_FRSM_TSK_ENTER(tskid);
	ercd = rsm_tsk(tskid);
	LOG_FRSM_TSK_LEAVE(ercd);
	return(ercd);
}

#endif /* __frsm_tsk */

/**
 *  ٱ
 */
#ifdef __dly_tsk

SYSCALL ER
dly_tsk(RELTIM dlytim)
{
	WINFO	winfo;
	TMEVTB	tmevtb;
	ER	ercd;

	LOG_DLY_TSK_ENTER(dlytim);
	CHECK_DISPATCH();
	CHECK_PAR(dlytim <= TMAX_RELTIM);

  retry:	
	T_ACQUIRE_LOCK_AND_CHECK_RUNNABLE();
	runtsk->tstat = TS_WAITING;
	make_non_runnable(runccb, runtsk);
	runtsk->winfo = &winfo;
	winfo.tmevtb = &tmevtb;
	tmevtb_enqueue(runccb, &tmevtb, dlytim, (CBACK) wait_tmout_ok, (VP) runtsk);
	LOG_TSKSTAT(runtsk);	
	RELEASE_LOCK(runccb->tsk_lock);
	dispatch();
	ercd = winfo.wercd;
	t_unlock_cpu();

  exit:
	LOG_DLY_TSK_LEAVE(ercd);
	return(ercd);
}

#endif /* __dly_tsk */
